/* -*- Mode: C++ -*-
 * $Id: GameTime.cc,v 1.2 2001/07/28 07:17:57 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: GameTime.cc
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-20
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/28 07:17:57 $
 *
 *
 * HISTORY
 *
 * $Log: GameTime.cc,v $
 * Revision 1.2  2001/07/28 07:17:57  mikbr212
 * Removed warnings!!
 *
 * Revision 1.1  2001/04/22 14:21:28  andbo928
 * Created.
 *
 * Revision 1.3  2001/04/17 14:55:27  andbo928
 * Created... might be buggy...
 *
 * Revision 1.2  2001/04/17 12:47:59  andbo928
 * It now uses worldstateview.
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.2  2000/11/08 17:54:20  olalu526
 * This fixes a LOT of problems.
 *
 * Revision 1.1  2000/09/20 14:42:48  frehe
 * Added the predicate GameTime that will return true iff the agent is the closest player to the ball.
 *
 */

/**
 * @file GameTime.cc
 * Implementation file for the predicate class GameTime.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: mikbr212 $
 * $Date: 2001/07/28 07:17:57 $
 */


#include "GameTime.h"
#include "view_constants.h"
#include "ViewManager.h"
#include "TVector.h"

RS_BEGIN_NAMESPACE

GameTime::GameTime()
  : Predicate(),
    worldView(NULL),
    parameterView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(worldView, WORLD_STATE_VIEW_ID);
}

GameTime::GameTime(const GameTime& obj)
  : Predicate(obj),
  worldView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(worldView, WORLD_STATE_VIEW_ID);
}

GameTime::~GameTime()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(WORLD_STATE_VIEW_ID);
}

void GameTime::setTime(int when)
{
  triggerTime = when;
}

GameTime& GameTime::operator= (const GameTime&)
{

  // Don't copy the worldView pointer.
  // Only the predicate parameters if any.
  return *this;
}

FuzzyBool GameTime::evaluate(const WorldState& ws)
{

  if(ws.getGameTime() == triggerTime)
    return TRUE;
  return FALSE;
}

FuzzyBool GameTime::evaluate()
{
  return evaluate(worldView->getWorldState());
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "GameTime.icc"
#endif
