/* -*- Mode: C++ -*-
 * $Id: ContinueIntention.cc,v 1.3 2001/07/26 18:58:49 mikbr212 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.3 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/26 18:58:49 $
 *
 *
 * HISTORY
 *
 * $Log: ContinueIntention.cc,v $
 * Revision 1.3  2001/07/26 18:58:49  mikbr212
 * Changed to new intention design
 *
 * Revision 1.2  2001/07/03 15:24:18  andbo928
 * Added utility methods.
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file ContinueIntention.cc
 * Implementation file for the class ContinueIntention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.3 $
 * $Author: mikbr212 $
 * $Date: 2001/07/26 18:58:49 $
 */


#include "ContinueIntention.h"

RS_BEGIN_NAMESPACE

/** Constructs an intention. 
 */
ContinueIntention::ContinueIntention()
  : Intention()
{
}

/** Destroys the intention. 
 */
ContinueIntention::~ContinueIntention() {
}

/** Generates a plan. 
 * Don't call this method! This method will generate an empty plan.
 */
Plan ContinueIntention::generatePlan() {
  return Plan(); // empty plan
}

/** Models a player. 
 * This intention moves the player forward in the direction
 * and speed it already has.
 * @param plr the player to be modelled
 */
void ContinueIntention::model(WorldState& world) {
  // Compensate for the player decay
  AgentState agent(world.getAgent());

  float power = agent.getSpeed().getLength() * 
    (1 - parameterView->SP_player_decay) / 
    parameterView->SP_dash_power_rate;

  // Just in case we didn't get the bodydir
  agent.setDirection(agent.getSpeed().getAngle());
  world.setAgent(agent);

  world.doDash(power);
}

/** Checks if the utility of this intention. Allways uSuck since
 * the agent should never have this intention.
 */
Intention::Utility ContinueIntention::utility() {
  return Intention::uSuck;
}

/** Checks if the model utility of this intention. Allways uNormal.
 */
Intention::Utility ContinueIntention::modelUtility(const WorldState&) {
  return Intention::uNormal;
}

/** Returns the persistence of this intention.
 * This intention is not persistent.
 * @return a float between zero (not persistent) and one (very persistent)
 */
float ContinueIntention::getPersistence() const {
  return 0; // Nope, not persistent
}

/** Returns the ID that identifies this (type of) intention.
 */
IntentionId ContinueIntention::getID() const {
  return CONTINUE_INTENTION_ID;
}

RS_END_NAMESPACE








