#include "FloatMath.h"
#include <stdio.h>

RS_BEGIN_NAMESPACE

/****************************************
 * Operation for binary float addition  *
 ****************************************/

std::vector<opType> OpFloatPlus::getArgumentTypes()
{
  std::vector<opType> aTypes;
  aTypes.insert(aTypes.end(), opFloat);
  aTypes.insert(aTypes.end(), opFloat);
  return aTypes;
}

float OpFloatPlus::getFloat()
{
  if(arguments.size() >= 2)
    return arguments[0]->getFloat() + arguments[1]->getFloat();
  return 0.0;
}

std::string OpFloatPlus::getOperationString()
{
  return "f+";
}

/*******************************************
 * Operation for binary float subtraction  *
 *******************************************/

std::vector<opType> OpFloatMinus::getArgumentTypes()
{
  std::vector<opType> aTypes;
  aTypes.insert(aTypes.end(), opFloat);
  aTypes.insert(aTypes.end(), opFloat);
  return aTypes;
}

float OpFloatMinus::getFloat()
{
  if(arguments.size() >= 2)
    return arguments[0]->getFloat() - arguments[1]->getFloat();
  return 0.0;
}

std::string OpFloatMinus::getOperationString()
{
  return "f-";
}

/**********************************************
 * Operation for binary float multiplication  *
 **********************************************/

std::vector<opType> OpFloatMultiply::getArgumentTypes()
{
  std::vector<opType> aTypes;
  aTypes.insert(aTypes.end(), opFloat);
  aTypes.insert(aTypes.end(), opFloat);
  return aTypes;
}

float OpFloatMultiply::getFloat()
{
  if(arguments.size() >= 2)
    return arguments[0]->getFloat() * arguments[1]->getFloat();
  return 0.0;
}

std::string OpFloatMultiply::getOperationString()
{
  return "f*";
}

/****************************************
 * Operation for binary float division  *
 ****************************************/

std::vector<opType> OpFloatDivide::getArgumentTypes()
{
  std::vector<opType> aTypes;
  aTypes.insert(aTypes.end(), opFloat);
  aTypes.insert(aTypes.end(), opFloat);
  return aTypes;
}

float OpFloatDivide::getFloat()
{
  if(arguments.size() >= 2)
    {
      float arg1 = arguments[1]->getFloat();
      if(arg1 == 0.0)
	{
	  return arguments[0]->getFloat() / (exp(-10));
	}
      else
	{
	  return arguments[0]->getFloat() / arg1;
	}
    }
  return 0.0;
}

std::string OpFloatDivide::getOperationString()
{
  return "f/";
}

/****************************************
 * Operation for float constants        *
 ****************************************/

OpFloatConstant::OpFloatConstant() {
  OpFloatConstant(0.0);
}

OpFloatConstant::OpFloatConstant(float val) {
  value = val;
}


std::vector<opType> OpFloatConstant::getArgumentTypes()
{
  std::vector<opType> aTypes;
  aTypes.clear();
  return aTypes;
}

float OpFloatConstant::getFloat()
{
  return value;
}

std::string OpFloatConstant::getOperationString()
{
  char buf[256];
  std::snprintf(buf,256, "f %f", value);
  return buf;
}

/****************************************
 * Operation for float arguments        *
 ****************************************/

std::vector<opType> OpFloatArg::getArgumentTypes()
{
  std::vector<opType> aTypes;
  aTypes.insert(aTypes.end(), opFloat);
  return aTypes;
}

float OpFloatArg::getFloat()
{
  int index = 0;
  if (arguments.size() > 0)
    index = floor(arguments[0]->getFloat());
  return getFloatArg(index); // getFloatArg handles out of bounds cases.
}

std::string OpFloatArg::getOperationString()
{
  return "farg";
}

RS_END_NAMESPACE









