#ifndef FUZZYCLEARBALL_H
#define FUZZYCLEARBALL_H


// Include the system header which definitions you use.
//#include <...>
//#include <...>

// Include the header files which definitions you use.
#include "Skill.h"

#include "CoordVector.h"
#include "CoordPoint.h"
#include "view_constants.h"
#include "ParameterView.h"
#include "MarkerView.h"
#include "TerritoryControlView.h"
#include "rs_log.h"
#include "KickBallToSpeedVector.h"
#include "WorldStateView.h"

RS_BEGIN_NAMESPACE

//class ParameterView;
//class MarkerView;
//class AgentView;
//class BallView;

/**
 * @class FuzzyClearBall FuzzyClearBall.h
 * This class implements the pass skill
 *
 * <!--
 * @pre HaveBall, visible players.
 * @post Ball hopefully at the closest passable player.
 * @invariant ?!?!
 *
 * @todo Finetune the skill.
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: andbo928 $
 * $Date: 2001/04/23 10:18:46 $
 */

class FuzzyClearBall : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  FuzzyClearBall();

  /** Destructor should clean up after the skill. */
  virtual ~FuzzyClearBall();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

protected:
  Vector calculateClearVector();
  bool isBallKickable();

  ParameterView* parameterView;
  MarkerView* markerView;
  WorldStateView* worldStateView;
  KickBallToSpeedVector kickBall;

  float clearSafeness;
  
  /** Hide copy-constructor. */
  FuzzyClearBall(const FuzzyClearBall&);
  /** Hide assignment operator. */
  FuzzyClearBall& operator= (const FuzzyClearBall&);
};


RS_END_NAMESPACE

#endif // !FUZZYCLEARBALL_H
