#ifndef FUZZYINTERCEPT_H
#define FUZZYINTERCEPT_H


// Include the system header which definitions you use.
//#include <...>
//#include <...>

// Include the header files which definitions you use.
#include "Skill.h"

#include "CoordVector.h"
#include "CoordPoint.h"
#include "view_constants.h"
#include "ParameterView.h"
#include "MarkerView.h"
#include "AgentView.h"
#include "BallView.h"
#include "TerritoryControlView.h"
#include "FuzzyPlayerView.h"
#include "FuzzyBallView.h"
#include "rs_log.h"
#include "KickBallToSpeedVectorMulti.h"
//#include "MoveToPositionFacingBall.h"

RS_BEGIN_NAMESPACE

//class ParameterView;
//class MarkerView;
//class AgentView;
//class BallView;

/**
 * @class FuzzyIntercept FuzzyIntercept.h
 * This class implements the pass skill
 *
 * <!--
 * @pre HaveBall, visible players.
 * @post Ball hopefully at the closest passable player.
 * @invariant ?!?!
 *
 * @todo Finetune the skill.
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: andbo928 $
 * $Date: 2001/04/13 18:16:33 $
 */

class FuzzyIntercept : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  FuzzyIntercept(int maxStp = 5);

  /** Destructor should clean up after the skill. */
  virtual ~FuzzyIntercept();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

protected:
  Point calculateInterceptPoint(int *t);
  Vector calculateDashVector(Point p);

  ParameterView* parameterView;
  MarkerView* markerView;
  AgentView* agentView;
  FuzzyBallView* ballView;
  FuzzyPlayerView* playerView;
  KickBallToSpeedVectorMulti kickBall;

  //  MoveToPositionFacingDirectionAvoidingObstacles mtpfd;

  int maxSteps;
  float clearSafeness;
  Point lastPos;
  Vector lastDash;
  bool  useLastPos;
  
  /** Hide copy-constructor. */
  FuzzyIntercept(const FuzzyIntercept&);
  /** Hide assignment operator. */
  FuzzyIntercept& operator= (const FuzzyIntercept&);
};


RS_END_NAMESPACE

#endif // !FUZZYINTERCEPT_H
