/* -*- Mode: C++ -*-
 * $Id: LookIntention.cc,v 1.12 2001/07/28 14:50:21 andbo928 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.12 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: andbo928 $ $Date: 2001/07/28 14:50:21 $
 *
 *
 * HISTORY
 *
 * $Log: LookIntention.cc,v $
 * Revision 1.12  2001/07/28 14:50:21  andbo928
 * Commented out nextSeeCycle stuff since it seems to be bugged.
 *
 * Revision 1.11  2001/07/28 14:44:44  sigge
 * Now looks more often
 *
 * Revision 1.10  2001/07/28 10:34:35  mikbr212
 * Fixed looking
 *
 * Revision 1.9  2001/07/12 12:55:40  andbo928
 * Patched, doesn't use lookview. For now.
 *
 * Revision 1.8  2001/07/06 16:28:20  mikbr212
 * Fixed nextSeeCycle-phenomena
 *
 * Revision 1.7  2001/07/06 15:37:29  mikbr212
 * Fixed for new utility-concept
 *
 * Revision 1.6  2001/07/06 14:09:58  mikbr212
 * New utility
 *
 * Revision 1.5  2001/07/06 08:36:07  mikbr212
 * Made the intention work (but not yet in the new design)
 *
 * Revision 1.4  2001/07/05 12:45:13  sigge
 * Quick hack for utility.
 *
 * Revision 1.3  2001/07/03 15:24:18  andbo928
 * Added utility methods.
 *
 * Revision 1.2  2001/07/01 14:16:00  sigge
 * Added Mickes code (Intentions)
 *
 * Revision 1.1  2001/06/19 21:35:10  mikbr212
 * Created stubs
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file LookIntention.cc
 * Implementation file for the class LookIntention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.12 $
 * $Author: andbo928 $
 * $Date: 2001/07/28 14:50:21 $
 */


#include "LookIntention.h"

#include "LookView.h"
#include "FindBall.h"

RS_BEGIN_NAMESPACE

/** Constructs an intention. 
 */
LookIntention::LookIntention()
  : Intention(),
    lookView(NULL)
{
  getView(lookView, LOOK_VIEW_ID);
}

/** Destroys the intention. 
 */
LookIntention::~LookIntention() {
  viewManager->releaseView(LOOK_VIEW_ID);
}

/* Generates a plan. 
 */
Plan LookIntention::generatePlan() {
  static FindBall findBall(0.3);
  
  // If the ball confidence is too low, then find it
  if(findBall.applicable())
    return findBall.generatePlan();

  // ... otherwise look in some other good direction
  AngleDeg lookDir(lookView->getAngle());
  AgentState newAgent(worldStateView->getWorldState().getAgent());
  newAgent.doTurnTo(lookDir.getDeg());
  newAgent.doTurnNeckTo(lookDir.getDeg());

  return Plan(newAgent.getStep());
}

/** Return @c the utility value of the intention.
 * The agent is considered.
 */
Intention::Utility LookIntention::utility() {

  // Check if I will see something next cycle
//    if((worldStateView->getNextSeeCycle() !=
//        worldStateView->getWorldState().getGameTime() + 1) &&
//       (worldStateView->getNextNextSeeCycle() !=
//        worldStateView->getWorldState().getGameTime() + 1))
//      return Intention::uSuck; // nope, then no use of looking...

  const BallState& ball(worldStateView->getWorldState().getBall());
  if(ball.getC() < 0.3)
    return Intention::uVeryGood;

  float lookVal = lookView->getMaxValue();
  if(lookVal > 0.8)
    return Intention::uVeryGood;
  else if(lookVal > 0.6)
    return Intention::uGood;
  else if(lookVal > 0.3)
    return Intention::uNormal;
  else if(lookVal > 0.25)
    return Intention::uBad;
  else 
    return Intention::uVeryBad;
}

/** Return the utility value of the intention.
 * Since this intention is not for modelling use, uSuck will be returned.
 */
Intention::Utility LookIntention::modelUtility(const WorldState&) {
  return Intention::uSuck;
}

/** Models a player. 
 * This intention is not for modelling. Does nothing.
 */
void LookIntention::model(WorldState&) {
  // do nothing
}

/** Returns the persistence of this intention.
 * This intention is not persistent at all. 
 * Usually the agent only needs one cycle.
 */
float LookIntention::getPersistence() const {
  return 0.0; // Not persistent at all. 
}

/** Returns the ID that identifies this (type of) intention.
 */
IntentionId LookIntention::getID() const {
  return LOOK_INTENTION_ID;
}

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "LookIntention.icc"
#endif







