/* -*- Mode: C++ -*-
 * $Id: LookIntention.h,v 1.6 2001/07/06 15:37:29 mikbr212 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.6 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/06 15:37:29 $
 *
 *
 * HISTORY
 *
 * $Log: LookIntention.h,v $
 * Revision 1.6  2001/07/06 15:37:29  mikbr212
 * Fixed for new utility-concept
 *
 * Revision 1.5  2001/07/06 14:09:58  mikbr212
 * New utility
 *
 * Revision 1.4  2001/07/06 08:36:07  mikbr212
 * Made the intention work (but not yet in the new design)
 *
 * Revision 1.3  2001/07/03 15:24:18  andbo928
 * Added utility methods.
 *
 * Revision 1.2  2001/07/01 14:16:00  sigge
 * Added Mickes code (Intentions)
 *
 * Revision 1.1  2001/06/19 21:35:10  mikbr212
 * Created stubs
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file Intention.h
 * Definition file for the class Intention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.6 $
 * $Author: mikbr212 $
 * $Date: 2001/07/06 15:37:29 $
 */

#ifndef LOOK_INTENTION_H
#define LOOK_INTENTION_H

#include "Intention.h"

RS_BEGIN_NAMESPACE

class LookView;

/** 
 * @class LookIntention LookIntention.h
 * This intention is applicable for a player that has the need
 * for looking around, that is if the player has low confidence in
 * certain objects. Since the knowledge that other players has is 
 * very hard to model, we don't model it.
 * Therefore this intention is just used for the agent itself.
 *
 * $Revision: 1.6 $
 * $Author: mikbr212 $
 * $Date: 2001/07/06 15:37:29 $
 */
class LookIntention : public Intention {
public:
  LookIntention();
  virtual ~LookIntention();

  virtual Intention::Utility utility();
  virtual Plan generatePlan();

  virtual Intention::Utility modelUtility(const WorldState&);
  virtual void model(WorldState&);

  virtual float getPersistence() const;

  virtual IntentionId getID() const;
  
protected:
  // The following views are inherited:
  //    ParameterView* parameterView;
  //    WorldStateView* worldStateView;
  //    PassTreeView* passTreeView;
  //    GameView* gameView;
  LookView* lookView;
  
private:
  /** Hide copy-constructor. */
  LookIntention(const LookIntention&);
  /** Hide assignment operator. */
  LookIntention& operator= (const LookIntention&);
};

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "LookIntention.icc"
#endif

#endif // LOOK_INTENTION_H
