#ifndef _POINTMATH_H_
#define _POINTMATH_H_


#include "Operation.h"
#include "CoordPoint.h"

RS_BEGIN_NAMESPACE

/****************************************
 * Operation for binary Point addition  *
 ****************************************/
class OpPointPlus : public PointOperation {
public:
  virtual Operation* clone() { return cloneOperation(this); }
  // Theese need to be overridden
   virtual std::vector<opType> getArgumentTypes();
   virtual Point getPoint();
protected:
  // Override!
  virtual std::string getOperationString();
};


/*******************************************
 * Operation for binary Point subtraction  *
 *******************************************/
class OpPointMinus : public PointOperation {
public:
  virtual Operation* clone() { return cloneOperation(this); }
  // Theese need to be overridden
   virtual std::vector<opType> getArgumentTypes();
   virtual Point getPoint();
protected:
  // Override!
  virtual std::string getOperationString();
};

/**********************************************
 * Operation for binary Point multiplication  *
 **********************************************/
class OpPointMultiply : public FloatOperation {
public:
  virtual Operation* clone() { return cloneOperation(this); }
  // Theese need to be overridden
  virtual std::vector<opType> getArgumentTypes();
  virtual float getFloat();
protected:
  // Override!
  virtual std::string getOperationString();
};

/**********************************************
 * Operation for binary Point scaling         *
 **********************************************/
class OpPointScale : public PointOperation {
public:
  virtual Operation* clone() { return cloneOperation(this); }
  // Theese need to be overridden
  virtual std::vector<opType> getArgumentTypes();
  virtual Point getPoint();
protected:
  // Override!
  virtual std::string getOperationString();
};

/****************************************
 * Operation for Point constants        *
 ****************************************/
class OpPointConstant : public PointOperation {
public:
  OpPointConstant();
  OpPointConstant(Point val);
  virtual Operation* clone() { return cloneOperation(this); }
  // Theese need to be overridden
  virtual std::vector<opType> getArgumentTypes();
  virtual Point getPoint();
protected:
  // Override!
  virtual std::string getOperationString();
private:
  Point value;
};


/**********************************************
 * Operation to get the length of a "point"   *
 **********************************************/
class OpPointLength : public FloatOperation {
public:
  virtual Operation* clone() { return cloneOperation(this); }
  // Theese need to be overridden
  virtual std::vector<opType> getArgumentTypes();
  virtual float getFloat();
protected:
  // Override!
  virtual std::string getOperationString();
};

/**********************************************
 * Operation to get the angle of a "point"    *
 **********************************************/
class OpPointAngle : public FloatOperation {
public:
  virtual Operation* clone() { return cloneOperation(this); }
  // Theese need to be overridden
  virtual std::vector<opType> getArgumentTypes();
  virtual float getFloat();
protected:
  // Override!
  virtual std::string getOperationString();
};

/**********************************************
 * Operation to get the x value of a point    *
 **********************************************/
class OpPointX : public FloatOperation {
public:
  virtual Operation* clone() { return cloneOperation(this); }
  // Theese need to be overridden
  virtual std::vector<opType> getArgumentTypes();
  virtual float getFloat();
protected:
  // Override!
  virtual std::string getOperationString();
};

/**********************************************
 * Operation to get the y value of a point    *
 **********************************************/
class OpPointY : public FloatOperation {
public:
  virtual Operation* clone() { return cloneOperation(this); }
  // Theese need to be overridden
  virtual std::vector<opType> getArgumentTypes();
  virtual float getFloat();
protected:
  // Override!
  virtual std::string getOperationString();
};

/**********************************************
 * Operation to construct a point from x/y    *
 **********************************************/
class OpPoint : public PointOperation {
public:
  // Theese need to be overridden
  virtual std::vector<opType> getArgumentTypes();
  virtual Point getPoint();
protected:
  // Override!
  virtual std::string getOperationString();
};

/**********************************************
 * Operation to construct a point from r/th   *
 **********************************************/
class OpPolPoint : public PointOperation {
public:
  virtual Operation* clone() { return cloneOperation(this); }
  // Theese need to be overridden
  virtual std::vector<opType> getArgumentTypes();
  virtual Point getPoint();
protected:
  // Override!
  virtual std::string getOperationString();
};


/****************************************
 * Operation for Point arguments        *
 ****************************************/
class OpPointArg : public PointOperation {
public:
  virtual Operation* clone() { return cloneOperation(this); }
  // Theese need to be overridden
   virtual std::vector<opType> getArgumentTypes();
   virtual Point getPoint();
protected:
  // Override!
  virtual std::string getOperationString();
};

RS_END_NAMESPACE

#endif
