/* -*- Mode: C++ -*-
 * $Id: ReceiveBallIntention.h,v 1.6 2001/07/06 14:33:15 andbo928 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.6 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: andbo928 $ $Date: 2001/07/06 14:33:15 $
 *
 *
 * HISTORY
 *
 * $Log: ReceiveBallIntention.h,v $
 * Revision 1.6  2001/07/06 14:33:15  andbo928
 * Fixed utility and stuff.
 *
 * Revision 1.5  2001/07/03 16:25:47  andbo928
 * Implemented the utility method.
 *
 * Revision 1.4  2001/07/03 15:24:19  andbo928
 * Added utility methods.
 *
 * Revision 1.3  2001/07/02 15:59:54  andbo928
 * New implementation, not fully functional yet.
 *
 * Revision 1.2  2001/07/01 14:16:00  sigge
 * Added Mickes code (Intentions)
 *
 * Revision 1.1  2001/06/19 21:35:10  mikbr212
 * Created stubs
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file Intention.h
 * Definition file for the class Intention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.6 $
 * $Author: andbo928 $
 * $Date: 2001/07/06 14:33:15 $
 */

#ifndef RECEIVE_BALL_INTENTION_H
#define RECEIVE_BALL_INTENTION_H

#include "Intention.h"
#include "Intercept.h"

RS_BEGIN_NAMESPACE

/** 
 * @class ReceiveBallIntention ReceiveBallIntention.h
 * This intention is used for a player that can receive 
 * (a moving) ball. It is also applicable if the player
 * can intercept the ball (by rushing to it). Note that
 * this intention is mainly applicable for fast balls, even
 * though it should be applicable for slow balls also if they
 * are close enough.
 *
 * $Revision: 1.6 $
 * $Author: andbo928 $
 * $Date: 2001/07/06 14:33:15 $
 */
class ReceiveBallIntention : public Intention {
public:
  ReceiveBallIntention();
  virtual ~ReceiveBallIntention();

  virtual Plan generatePlan();
  virtual void model(WorldState&);

  // Uncomment if the agent should be threated specially
  //virtual Utility utility();
  virtual Utility utility();


  /** Return @c the utility value of the intention.
   * This should be a light weight method, since this
   * will be called for all other players.
   */
  virtual Utility modelUtility(const WorldState&);

  virtual float getPersistence() const;

  virtual IntentionId getID() const;
  
protected:
  // Uncomment either chooseSkill or generatePlan
  //  virtual Skill* chooseSkill();
  //  virtual Plan generatePlan();
  virtual double evalWorldState(const WorldState& worldState);
  float ballIsHeadedTo(const PlayerState&); 

  double exaggerate(double d);
  void findBetter(int plrTime, 
		  const BallState& ball, 
		  const vector<PlayerState>& plrs, 
		  int& better, 
		  int& similar);

  // The following views are inherited:
  //    ParameterView* parameterView;
  //    WorldStateView* worldStateView;
  //    PassTreeView* passTreeView;
  //    GameView* gameView;
  
private:
  /** Hide copy-constructor. */
  ReceiveBallIntention(const ReceiveBallIntention&);
  /** Hide assignment operator. */
  ReceiveBallIntention& operator= (const ReceiveBallIntention&);
};

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ReceiveBallIntention.icc"
#endif

#endif // RECEIVE_BALL_INTENTION_H
