/* -*- Mode: C++ -*-
 * $Id: TakeBallIntention.cc,v 1.30 2001/07/31 14:11:57 andbo928 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.30 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: andbo928 $ $Date: 2001/07/31 14:11:57 $
 *
 *
 * HISTORY
 *
 * $Log: TakeBallIntention.cc,v $
 * Revision 1.30  2001/07/31 14:11:57  andbo928
 * Fixed so it doesn't get the ball if a teammate can kick.
 *
 * Revision 1.29  2001/07/30 17:20:35  andbo928
 * Now ignores goalie when calculating teammates. Despammed.
 *
 * Revision 1.28  2001/07/30 16:50:14  andbo928
 * Improved handling. Dashes for home if needed.
 *
 * Revision 1.27  2001/07/30 12:18:21  mikbr212
 * Moved some code from the old gotoball to here
 *
 * Revision 1.26  2001/07/30 11:10:48  mikbr212
 * Added intercept (as it was before)
 *
 * Revision 1.25  2001/07/29 17:07:27  mikbr212
 * cleanup
 *
 * Revision 1.24  2001/07/29 14:19:35  mikbr212
 * Fixed look
 *
 * Revision 1.23  2001/07/28 16:05:04  mikbr212
 * More agressive defence
 *
 * Revision 1.22  2001/07/28 14:50:50  andbo928
 * Requires less ball confidence.
 *
 * Revision 1.21  2001/07/28 07:18:12  mikbr212
 * Removed warnings!!
 *
 * Revision 1.20  2001/07/26 15:43:42  mikbr212
 * Fixed some includes
 *
 * Revision 1.19  2001/07/09 16:52:52  andbo928
 * Minor changes.
 *
 * Revision 1.18  2001/07/06 16:30:11  andbo928
 * Despammed.
 *
 * Revision 1.17  2001/07/06 14:46:03  andbo928
 * Fixed model-method.
 *
 * Revision 1.16  2001/07/06 14:31:41  sigge
 * uSucks -> uSuck
 *
 * Revision 1.15  2001/07/06 14:07:11  andbo928
 * Added modelutility
 *
 * Revision 1.14  2001/07/06 14:03:09  andbo928
 * Added modelutility
 *
 * Revision 1.13  2001/07/06 12:37:39  andbo928
 * Added utility comment and uSucks util value.
 *
 * Revision 1.12  2001/07/06 08:59:55  mikbr212
 * Made the some things work (but not yet in the new design)
 *
 * Revision 1.11  2001/07/05 12:44:34  andbo928
 * Now considers opponents when calculating utility.
 *
 * Revision 1.10  2001/07/05 12:23:31  andbo928
 * More agressive play.
 *
 * Revision 1.9  2001/07/04 16:57:12  andbo928
 * Implemented utility method roughly.
 *
 * Revision 1.8  2001/07/04 16:44:48  andbo928
 * Works somewhat better now, not perfect yet.
 *
 * Revision 1.7  2001/07/04 14:27:00  andbo928
 * Added intercept.
 *
 * Revision 1.6  2001/07/03 15:24:19  andbo928
 * Added utility methods.
 *
 * Revision 1.5  2001/07/03 12:43:47  sigge
 * Changed enum names in MoveSpeed
 *
 * Revision 1.4  2001/07/02 12:41:08  sigge
 * MoveToPosition (and GotoBall) now supports setSpeed
 *
 * Revision 1.3  2001/06/19 16:35:47  mikbr212
 * Added comments
 *
 * Revision 1.2  2001/06/19 15:21:51  mikbr212
 * Changed conf in ball in applicable
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file TakeBallIntention.cc
 * Implementation file for the class TakeBallIntention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.30 $
 * $Author: andbo928 $
 * $Date: 2001/07/31 14:11:57 $
 */

#include "TakeBallIntention.h"
#include "GotoBall.h"
#include "Intercept.h"
#include "GameView.h"

RS_BEGIN_NAMESPACE
/** Constructs an intention. 
 */
TakeBallIntention::TakeBallIntention()
  : Intention()
{
}

/** Destroys the intention. 
 */
TakeBallIntention::~TakeBallIntention() {
}

/** Returns the position between the ball and goal. 
 */
TVector TakeBallIntention::getCutOffGoalAnglePosition(Team team) {
  const BallState& ball(worldStateView->getWorldState().getBall());
  TVector goalVec;
  if(team == T_OUR_TEAM) {
    goalVec = TVector(-52.5,0) - ball.getPosition();
  } else {
    goalVec = TVector(52.5,0) - ball.getPosition();
  }    
  const float ballDist(3.5);
  goalVec.setLength(ballDist);
  return ball.getPosition() + goalVec;
}

/** Underestimate a value, max(0.0, (d-1)/1.2)
 */
double TakeBallIntention::underestimate(double d)
{
  if(d > 2.0)
    return (d - 2.0)/1.1;
  else
    return 0.0;
}

/** Overestimates a value, default is return d*1.2 + 1
 */
double TakeBallIntention::overestimate(double d)
{
  return d*1.1 + 2.0;
}

void TakeBallIntention::findBetter(int plrTime,
				   bool ignoreGoalie,
				   const BallState& ball, 
				   const vector<PlayerState>& plrs, 
				   int& better, int& similar)
{ 
  int myUnderTime = int(floor(underestimate(plrTime)));
  int myOverTime = int(ceil(overestimate(plrTime)));
//    cout << "u: " << myUnderTime << ", o:" << myOverTime << endl;
  better = 0;
  similar = 0;
  vector<PlayerState>::const_iterator it;
  for(it = plrs.begin(); it != plrs.end(); it++)
  {
    if( !(ignoreGoalie && it->getUniformNumber() == 1) )
    {
      int curTime = it->getInterceptTime(ball);
      if(curTime < myOverTime && curTime > myUnderTime)
      {
	//        cout << "sim: " << curTime << endl;
	similar++;
      }
      
      if(curTime < myUnderTime)
      {
	better++;
      }
    }
  }
}

/** Chooses a skill. It works like a mini-decision maker.
 * If an opponent is about as close to the ball as I, then
 * run as fast as possible. Otherwise, if the opponent is
 * much closer or furter away, then just walk towards the ball
 * to save stamina.
 */
Skill* TakeBallIntention::chooseSkill() {
//    cout << "TakeBall: ";
  static GotoBall gotoBall;
  static MoveToPosition moveToPosition;
  static Intercept intercept(20);
  static Intercept shortIntercept(5);
  const WorldState ws(worldStateView->getWorldState());
  const std::vector<PlayerState> op(ws.getOpponents());
  const AgentState agent(ws.getAgent());
  const BallState& ball(ws.getBall());
  
  TVector agentToBall(ball.getPosition() - agent.getPosition());
  TVector delta = 
    ws.getAgent().getPosition()
    - ws.getBall().getPosition();
  double ballDist = delta.getLength();

  int myTime = agent.getInterceptTime(ws.getBall());
  int numBetter;
  int numSimilar;
  findBetter(myTime,
	     false,
	     ws.getBall(),
	     ws.getOpponents(),
	     numBetter,
	     numSimilar);

  if(gameView->isOurBall()) 
  {
    // Game Stopped, our ball.
//      cout << "Our ball, gotoBall" << endl;
    gotoBall.setSpeed(MoveToPosition::msGainStamina);
    gotoBall.setLookAtBall(ball.getC() < 0.7);
    return &gotoBall;
  }

  else if(numBetter == 0 && numSimilar == 0) 
  {
    // No need to hurry that much
    if(intercept.applicable()) 
    {
//        cout << "Intercept - noThreat." << endl;
      return &intercept;
    }
//      cout << "GotoBall - noThreat." << endl;
    gotoBall.setSpeed(MoveToPosition::msNormal);
    gotoBall.setLookAtBall(ball.getC() < 0.7);
    return &gotoBall;
  } 

  else if(numBetter > 0) 
  {
    // Time for some recoding here..
    const vector<PlayerState>& op = ws.getOpponents();
    vector<PlayerState>::const_iterator itOp;
    bool kickerFound = false;
    TVector kickerPos;
    for(itOp = op.begin(); itOp != op.end(); itOp++)
    {
      TVector delta = ball.getPosition() - itOp->getPosition();
      if(delta.getLength() < parameterView->SP_kickable_area*2)
      {
	kickerFound = true;
	kickerPos = itOp->getPosition();
	break;
      }
    }

    if(kickerFound)
    {
      TVector bToK = TVector(-52.5,0) - kickerPos;

      bToK.setLength(minMax(0.0F, 
			    (float) bToK.getLength(),
			    5.0F));
      TVector stressPosition = kickerPos + bToK;
      
//        if(ball.getPosition().getX() < -5 &&
//  	 agentToBall.getX() < 0) { 
      if(ball.getPosition().getX() < (agent.getPosition().getX() - 4.0))
      {
	// Try to get in front of mr. Breakaway guy.
//  	cout << "Stress - get home!" << endl;
	moveToPosition.setPosition(TVector(-52.5,0));
	moveToPosition.setSpeed(MoveToPosition::msFast);
      }
      else if(kickerPos.getX() < (agent.getPosition().getX() - 5))
      {
//  	cout << "Stress - get in front of him!" << endl;
	moveToPosition.setPosition(stressPosition);
	moveToPosition.setSpeed(MoveToPosition::msFast);
      } else {
//  	cout << "Stress - preparing!" << endl;
	moveToPosition.setPosition(stressPosition);
	moveToPosition.setSpeed(MoveToPosition::msKeepStamina);
      }

      if(ball.getC() < 0.5) {
	moveToPosition.setLookAtBall(true);
      } else {
	moveToPosition.setLookAtBall(false);
      }

      return &moveToPosition;
    }
    else
    {

      if(ball.getPosition().getX() < (agent.getPosition().getX() - 4.0))
      {
	// Try to get in front of mr. Breakaway guy.
//  	cout << "Gambling - get home!" << endl;
	moveToPosition.setPosition(TVector(-52.5,0));
	moveToPosition.setSpeed(MoveToPosition::msFast);
	moveToPosition.setLookAtBall(true);
	return &moveToPosition;
      }
      if(shortIntercept.applicable())
      {
//  	cout << "Gambling - intercept." << endl;
	return &shortIntercept;
      }
      else
      {
//  	cout << "Gambling - gotoball." << endl;
	gotoBall.setSpeed(MoveToPosition::msFast);
	return &gotoBall;
      }
    }

    // At least one opponent is better then us.

    // I can't take the ball :(
    // Just make the opponent nervous
  } 

  else 
  {
    // No opponent is better.
    // One or more opponent is similar.

    if(intercept.applicable()) 
    {
//        cout << "Threat, Intercept" << endl;
      return &intercept;
    }

//      cout << "Threat, gotoBall" << endl;
    if(ballDist < 30.0) {
      if(ball.getPosition().getX() < -5 &&
	 agentToBall.getX() < 0) { 
	gotoBall.setSpeed(MoveToPosition::msVeryFast);
      } else {
	gotoBall.setSpeed(MoveToPosition::msFast);
      }
      gotoBall.setLookAtBall(true);
    } else {
      gotoBall.setSpeed(MoveToPosition::msNormal);
      gotoBall.setLookAtBall(ball.getC() < 0.8);
    } 
    return &gotoBall;
  }


  cout << "Que, gotoBall" << endl;
  return &gotoBall;
}

/** Models a player. 
 * This intention simply calls plr.doNull().
 * @param plr the player to be modelled
 */
void TakeBallIntention::model(WorldState& ws) {
  ws.doNull(); // Do nothing
}

Intention::Utility TakeBallIntention::modelUtility(const WorldState& ws)
{
  if(ws.getBall().getC() < 0.3)
    return Intention::uSuck;

  TVector delta = ws.getBall().getPosition() - ws.getAgent().getPosition();
  float dist = delta.getLength();
  if(dist < 5.0)
    return Intention::uVeryGood;
  if(dist < 10.0)
    return Intention::uGood;
  if(dist < 15.0)
    return Intention::uNormal;
  if(dist < 25.0)
    return Intention::uBad;

  return Intention::uVeryBad;
}

/** Checks if the utility of this intention.
 * uSucks    - The ball confidence is to low.
 * uVeryBad  - At least two teammates are closer to the ball.
 * uBad      - One teammate is close to the ball
 *             or at least two teammates are at similar distances
 *             or at least two opponents are closer to the ball.
 * uNormal   - One teammate is at similar distance
 *             or on opponent is better.
 * uGood     - At least one opponent is at similar distance.
 * uVeryGood - No teammates are closer or at similar distances
 *             and no opponents are closer or at similar distances.
 */
Intention::Utility TakeBallIntention::utility() {
  WorldState ws = worldStateView->getWorldState();
  ws.reduceC(0.3);
  const AgentState& agent = ws.getAgent();

  if(ws.getBall().getC() < 0.5)
    return Intention::uSuck;

  int myTime = agent.getInterceptTime(ws.getBall());
  int numBetter;
  int numSimilar;
  findBetter(myTime,
	     true,
	     ws.getBall(),
	     ws.getTeammates(),
	     numBetter,
	     numSimilar);

  int numOpponentsBetter;
  int numOpponentsSimilar;
  findBetter(myTime,
	     false,
	     ws.getBall(),
	     ws.getOpponents(),
	     numOpponentsBetter,
	     numOpponentsSimilar);

  const vector<PlayerState>& tm = ws.getTeammates();
  bool teammateCanKick = false;
  for(int i = 0; i < tm.size(); i++)
  {
    TVector delta = tm[i].getPosition() - ws.getBall().getPosition();
    if(delta.getLength() < 1.5*parameterView->SP_kickable_area)
    {
      teammateCanKick = true;
      break;
    }
  }

//    if(agent.getUniformNumber() == 11)
//    {
//      cout << "Tm.size: " << ws.getTeammates().size() << " ,";
//      cout << "Better: " << numBetter << ", Similar: " << numSimilar << endl;
//    }

  // Check if it's their ball.
  if(gameView->isTheirBall())
    return Intention::uSuck;

  // Check if it's our ball.
  if(gameView->isOurBall())
  {
    if(numBetter > 0)
      return Intention::uBad;
    if(numSimilar > 1)
      return Intention::uNormal;
    if(numSimilar > 0)
      return Intention::uGood;

    return Intention::uVeryGood;
  }

  if(numBetter > 1 || teammateCanKick)
  {
    // Atleast two teammates are better.
//      cout << "VeryBad" << endl;
    return Intention::uVeryBad;
  }

  if(numBetter > 0 || numSimilar > 1) // || numOpponentsBetter > 1)
  {
    // Atleast one teammate is better
    // or at least two teammates are at similar distances
    // or at least two opponents are better.
//      cout << "Bad" << endl;
    return Intention::uBad;
  }

  if(numSimilar > 0) // || numOpponentsBetter > 0)
  {
    // At most one teammate is at similar distance
    // or at least one opponent is better.
//      cout << "Normal" << endl;
    return Intention::uNormal;
  }

  if(numOpponentsSimilar > 1 || numOpponentsSimilar > 0)
  {
    // At least one opponent is at a similar distance.
//      cout << "Good" << endl;
    return Intention::uGood;
  }

  // I am closest to the ball (no opponents nor teammates are closer or
  // similar).
//    cout << "VeryGood" << endl;
  return Intention::uVeryGood;
}

/** Returns the persistence of this intention.
 * This intention is not that persistent.
 * @return a float between zero (not persistent) and one (very persistent)
 */
float TakeBallIntention::getPersistence() const {
  return 0.4; 
}

/** Returns the ID that identifies this (type of) intention.
 */
IntentionId TakeBallIntention::getID() const {
  return TAKE_BALL_INTENTION_ID;
}

RS_END_NAMESPACE








