/* -*- Mode: C++ -*-
 * $Id: ZoneDefenceIntention.cc,v 1.15 2001/07/29 15:09:40 mikbr212 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.15 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/29 15:09:40 $
 *
 *
 * HISTORY
 *
 * $Log: ZoneDefenceIntention.cc,v $
 * Revision 1.15  2001/07/29 15:09:40  mikbr212
 * more aggresive defenders
 *
 * Revision 1.14  2001/07/29 14:21:02  mikbr212
 * Fixed move in case D, maybe?
 *
 * Revision 1.13  2001/07/28 14:32:48  sigge
 * ZoneDefence TNG
 *
 * Revision 1.11  2001/07/28 07:18:12  mikbr212
 * Removed warnings!!
 *
 * Revision 1.10  2001/07/27 13:18:11  sigge
 * Fixed offside bug.
 *
 * Revision 1.9  2001/07/26 13:47:05  sigge
 * Offside protection.
 *
 * Revision 1.8  2001/07/06 14:13:32  sigge
 * New model/utility stuff
 *
 * Revision 1.7  2001/07/06 13:55:51  sigge
 * New model/utility stuff
 *
 * Revision 1.6  2001/07/05 12:41:59  sigge
 * Don't mark players that have the ball
 *
 * Revision 1.5  2001/07/04 13:32:49  sigge
 * Separated victim selector and improved utility function.
 *
 * Revision 1.4  2001/07/03 15:24:19  andbo928
 * Added utility methods.
 *
 * Revision 1.3  2001/07/03 12:45:47  sigge
 * Added simple Role mechanism and did a basic ZoneDefence
 *
 * Revision 1.2  2001/07/02 15:59:26  andbo928
 * Cosmetic changes.
 *
 * Revision 1.1  2001/06/19 21:35:10  mikbr212
 * Created stubs
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file ZoneDefenceIntention.cc
 * Implementation file for the class ZoneDefenceIntention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.15 $
 * $Author: mikbr212 $
 * $Date: 2001/07/29 15:09:40 $
 */

#include "ZoneDefenceIntention.h"

#include "FindBall.h"
#include "LookView.h"
#include "MarkPosition.h"
#include "MoveToPosition.h"
#include "PlanContainer.h"
#include "Role.h"
#include "WorldStateView.h"

RS_BEGIN_NAMESPACE

/** Constructs an intention. 
 */
ZoneDefenceIntention::ZoneDefenceIntention()
  : Intention(),
    lookView(NULL)
{
  getView(lookView, LOOK_VIEW_ID);
}

/** Destroys the intention. 
 */
ZoneDefenceIntention::~ZoneDefenceIntention() {
}

// Uncomment either chooseSkill or generatePlan
/* Generates a plan. 
 * Returns an empty plan.
 *
 Plan ZoneDefenceIntention::generatePlan() {
 return Plan(); // Your plan
 }
*/

/** Selects the opponent in the zone to mark. It should choose the
 *  most dangerous one, but currently just selects the one closest to
 *  the goal.
 *
 *  Returns -1 if no opponent was found in the zone.
 */
int ZoneDefenceIntention::selectVictim(const WorldState& ws)
{
  static Role role;
  TVector center = role.getHomePosition();
  float radius = role.getZoneRadius();

  // Skip opponents that have the ball
  const vector<PlayerState> opponents = ws.getOpponents();
  const BallState ball = ws.getBall();
  int closestNumber = -1;
  float closestDistance = 10000.0;
  const TVector goal(-52.5, 0);
  for(unsigned int i=0; i<opponents.size(); i++)
  {
    float distanceToCenter = (opponents[i].getPosition()-center).getLength();
    float distanceToGoal = (opponents[i].getPosition()-goal).getLength();
    if(distanceToCenter < radius
       && distanceToGoal < closestDistance
       && !opponents[i].isBallKickable(ball) // FIXME: Closest interceptor?
       )
    {
      closestNumber = i;
      closestDistance = distanceToGoal;
    }
  }

  return closestNumber;
}

// Uncomment either chooseSkill or generatePlan
/** Chooses a skill. It works like a mini-decision maker.
 *
 * If there are opponents in the zone, mark the most dangerous one
 * (selection done by selectVictim).
 *
 * If a dangerous opponent (one carrying the ball) is approaching the
 * zone, meet up with him. Combine this with marking, if marking is
 * done.
 *
 * If both marking and meeting-up-with-opponent are done
 * simultaneously, position on the line from the mark victim to the
 * ball carrier. If not, then get closer to the carrier.
 *
 */
Skill* ZoneDefenceIntention::chooseSkill()
{
  static MoveToPosition moveToPosition;
  static MarkPosition markPosition;
  static Role role;
  static FindBall findBall;

  const WorldState& ws = worldStateView->getWorldState();
  const AgentState& agent(ws.getAgent());
  TVector center = role.getHomePosition();

  const vector<PlayerState> opponents = ws.getOpponents();
  int victimNumber = selectVictim(ws);

  int theirInterceptionTime = statisticsView->getTheirInterceptionTime();
  int ourInterceptionTime = statisticsView->getTheirInterceptionTime();
  const int haveBallThreshold = 5;
  const float closeThreshold = 30.0;

  const TVector goal(-52.5, 0);
  const TVector ballPos = ws.getBall().getPosition();
  const TLine ballToGoal(ballPos, goal);
  const TVector threat =
    statisticsView->getInterceptionBallState().getPosition();
  const TLine threatToGoal(threat, goal);

  // If they are have the lowest icepttime and it is below
  // haveBallThreshold, and the threat is closer than closeThreshold
  // to the center of the zone, we consider him to have the ball
  if(theirInterceptionTime < ourInterceptionTime
     && theirInterceptionTime <= haveBallThreshold
     && (threat - center).getLength() < closeThreshold)
  {
    // Meet up with threat
    if(victimNumber == -1)
    {
      // ********************************
      // Case A: Just meet up with threat
      // ********************************

      // Go to the closest point from center to threatToGoal
      TVector position = threatToGoal.getDistance(center);
      moveToPosition.setPosition(position);
      moveToPosition.setSpeed(MoveToPosition::msNormal);
      moveToPosition.setDistance(1.0);
      moveToPosition.setLookAtBall(true);

      if(moveToPosition.applicable())
	return &moveToPosition;
      else
	return &findBall;
    }

    else
    {
      // *********************************************
      // Case B: Meet up with threat _and_ mark player
      // *********************************************

      // Change the distance to the mark target according to:
      // oppDist	markDist
      // 30		5
      // 20		10
      // 10		5
      // This should be accomplished with max(5, 10 - 0.5*abs(oppDist-20))

      const TVector target = opponents[victimNumber].getPosition();
      const float oppDist = (target-threat).getLength();
      markPosition.setDistance( max(5.0, 10 - 0.5*abs(oppDist-20)) );
      return &markPosition;
    }
  }

  else
  {
    // Just mark the victim
    if(victimNumber == -1)
      {
      // *******************************************************
      // Case C: No victim, go to the center and do some looking
      // *******************************************************

      moveToPosition.setPosition(center);
      // Have a tolerance of 2 meters
      moveToPosition.setDistance(2.0);
      if(moveToPosition.applicable()) {
	// Run down fast, and up slowly
	float dist = center.getX() - agent.getPosition().getX();
	if(dist < -7) {
	  moveToPosition.setSpeed(MoveToPosition::msFast);
	} else if(dist < -3) {
	  moveToPosition.setSpeed(MoveToPosition::msNormal);
	} else {
	  moveToPosition.setSpeed(MoveToPosition::msGainStamina);
	}
	
	return &moveToPosition;
      } else {
	static PlanContainer planConainer;
	// I'm already at a good position, do some looking
	// If the ball confidence is too low, then find it
	if(findBall.applicable())
	  return &findBall;

	// ... otherwise look in some other good direction
	AngleDeg lookDir(lookView->getAngle());
	AgentState newAgent(worldStateView->getWorldState().getAgent());
	newAgent.doTurnTo(lookDir.getDeg());
	newAgent.doTurnNeckTo(lookDir.getDeg());

	planConainer.setPlan(Plan(newAgent.getStep()));
	return &planConainer;
      }
    }

    else
    {
      // ***********************
      // Case D: Mark the victim
      // ***********************
      TVector markPos(opponents[victimNumber].getPosition());
      TVector toMarkPos(markPos - agent.getPosition());
      if(toMarkPos.getLength() > 6) {
	moveToPosition.setPosition(markPos);
	if(toMarkPos.getX() < - 1)
	  moveToPosition.setSpeed(MoveToPosition::msFast);
	else
	  moveToPosition.setSpeed(MoveToPosition::msKeepStamina);
	return &moveToPosition;
      }
      
      markPosition.setPosition(opponents[victimNumber].getPosition());
      return &markPosition;
    }
  }
  return NULL; // not reached anyway...
}

/** Models a player. 
 * This intention simply calls plr.doNull().
 * @param plr the player to be modelled
 */
void ZoneDefenceIntention::model(WorldState&) {
  // FIXME: Proper modelling should be done
}

/** Checks if the utility of this intention.
 *  uBad: Home position is too far from the ball, marking is unnessecary
 *  uNormal: No one to mark, but might still be a good idea to be here
 *  uGood: Player to mark exists.
 */
Intention::Utility ZoneDefenceIntention::utility()
{
  TVector ballPos = worldStateView->getWorldState().getBall().getPosition();
  static Role role;
  TVector homePos = role.getHomePosition();

  const float defenceTooFar = 40.0;
  if( (ballPos - homePos).getLength() > defenceTooFar )
    return Intention::uBad;
  if(selectVictim(worldStateView->getWorldState()) == -1)
    return Intention::uNormal;	// No one to mark
  else
    return Intention::uGood;
}

Intention::Utility ZoneDefenceIntention::modelUtility(const WorldState& ws)
{
  // This is cheap enough, so it will be done the same way as the
  // regular utility
  TVector ballPos = ws.getBall().getPosition();
  static Role role;
  TVector homePos = role.getHomePosition();
  const float defenceTooFar = 40.0;
  if( (ballPos - homePos).getLength() > defenceTooFar )
    return Intention::uBad;
  if(selectVictim(ws) == -1)
    return Intention::uNormal;	// No one to mark
  else
    return Intention::uGood;
}

/** Returns the persistence of this intention.
 * This intention is very persistent.
 * @return a float between zero (not persistent) and one (very persistent)
 */
float ZoneDefenceIntention::getPersistence() const {
  return 1.0; // Very persistent
}

/** Returns the ID that identifies this (type of) intention.
 */
IntentionId ZoneDefenceIntention::getID() const {
  return ZONE_DEFENCE_INTENTION_ID;
}

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "ZoneDefenceIntention.icc"
#endif
