/* -*- Mode: C++ -*-
 * $Id: ZoneDefenceIntention.h,v 1.7 2001/07/28 14:32:48 sigge Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.7 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: sigge $ $Date: 2001/07/28 14:32:48 $
 *
 *
 * HISTORY
 *
 * $Log: ZoneDefenceIntention.h,v $
 * Revision 1.7  2001/07/28 14:32:48  sigge
 * ZoneDefence TNG
 *
 * Revision 1.6  2001/07/06 14:13:32  sigge
 * New model/utility stuff
 *
 * Revision 1.5  2001/07/06 13:55:51  sigge
 * New model/utility stuff
 *
 * Revision 1.4  2001/07/04 13:32:49  sigge
 * Separated victim selector and improved utility function.
 *
 * Revision 1.3  2001/07/03 15:24:19  andbo928
 * Added utility methods.
 *
 * Revision 1.2  2001/07/03 12:45:47  sigge
 * Added simple Role mechanism and did a basic ZoneDefence
 *
 * Revision 1.1  2001/06/19 21:35:10  mikbr212
 * Created stubs
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file Intention.h
 * Definition file for the class Intention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.7 $
 * $Author: sigge $
 * $Date: 2001/07/28 14:32:48 $
 */

#ifndef ZONE_DEFENCE_INTENTION_H
#define ZONE_DEFENCE_INTENTION_H

#include "Intention.h"

RS_BEGIN_NAMESPACE

class LookView;

/** 
 * @class ZoneDefenceIntention ZoneDefenceIntention.h
 * This intention is applicable when the player wants
 * to defend a zone. It will then mark opponents within
 * this zone and meet balls that are directed into the zone.
 *
 * $Revision: 1.7 $
 * $Author: sigge $
 * $Date: 2001/07/28 14:32:48 $
 */
class ZoneDefenceIntention : public Intention {
public:
  ZoneDefenceIntention();
  virtual ~ZoneDefenceIntention();

  // Uncomment either chooseSkill or generatePlan
  //  virtual Plan generatePlan();
  virtual void model(WorldState&);

  // Uncomment if the agent should be threated specially
  //virtual Utility utility();
  virtual Utility utility();
  virtual Utility modelUtility(const WorldState&);

  virtual float getPersistence() const;

  virtual IntentionId getID() const;
  
protected:
  // Uncomment either chooseSkill or generatePlan
  virtual Skill* chooseSkill();

  // Select the victim that should be marked
  int selectVictim(const WorldState& ws);

  // This method should not need to be overloaded
  //  virtual Plan getPlan(bool clear=false);

  // The following method is inherited:
  //  virtual Plan getPlan(bool clear=false);
  
  // The following views are inherited:
  //    ParameterView* parameterView;
  //    WorldStateView* worldStateView;
  //    PassTreeView* passTreeView;
  //    GameView* gameView;

  LookView* lookView;

private:
  /** Hide copy-constructor. */
  ZoneDefenceIntention(const ZoneDefenceIntention&);
  /** Hide assignment operator. */
  ZoneDefenceIntention& operator= (const ZoneDefenceIntention&);
};

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "ZoneDefenceIntention.icc"
#endif

#endif // ZONE_DEFENCE_INTENTION_H
