/* -*- Mode: C++ -*-
 * $Id: FCFooDecision.cc,v 1.29 2001/07/06 10:01:49 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FCFooDecision.cc
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.29 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/06 10:01:49 $
 *
 *
 * HISTORY
 *
 * $Log: FCFooDecision.cc,v $
 * Revision 1.29  2001/07/06 10:01:49  mikbr212
 * Merged updates and resolved conflicts (a lot)
 *
 * Revision 1.28  2001/04/24 17:46:23  sigge
 * Made goalie guard the goal more closely, and check for interception timeout
 *
 * Revision 1.27  2001/04/24 17:10:00  mikbr212
 * updated a lot
 *
 * Revision 1.26  2001/04/24 09:54:47  sigge
 * Moved communication policy to communication view. Communication view no longer erases messages not picked up from the incoming queue in order to be useful to skills.
 *
 * Revision 1.25  2001/04/23 15:11:50  sigge
 * Spam removal.
 *
 * Revision 1.24  2001/04/23 15:05:34  sigge
 * Goalie now uses narrow vision
 *
 * Revision 1.23  2001/04/23 12:47:18  sigge
 * Dynamic use of MarkPosition.
 *
 * Revision 1.22  2001/04/23 11:29:27  sigge
 * Now has two different setups after respective teams goal.
 *
 * Revision 1.21  2001/04/23 11:15:59  sigge
 * Intercept now has a maxSteps attribute.
 *
 * Revision 1.20  2001/04/23 10:54:31  andbo928
 * Cleaned up.
 *
 * Revision 1.19  2001/04/23 10:20:39  sigge
 * LoadConfig now uses TVector.
 *
 * Revision 1.18  2001/04/23 10:05:12  mikbr212
 * removed old views
 *
 * Revision 1.17  2001/04/23 09:34:24  sigge
 * Now uses Dribble, LookView and teleports when appropriate.
 *
 * Revision 1.16  2001/04/22 16:56:21  mikbr212
 * removed old views such as agentView etc..
 *
 * Revision 1.15  2001/04/21 14:44:37  sigge
 * Removed a whole bunch of skills from the Makefiles. Now the agent should be free of fuzzy views.
 *
 * Revision 1.14  2001/04/20 18:01:58  sigge
 * Spam removal.
 *
 * Revision 1.13  2001/04/20 18:00:47  sigge
 * Goalie now does clearball. decide() avoids deciding multiple times per cycle (aparent after play mode changes, it seems). Some old commented stuff removed.
 *
 * Revision 1.12  2001/04/20 16:21:58  sigge
 * Replaced getPlayerMode with getPlayerModes.
 *
 * Revision 1.11  2001/04/20 14:45:05  sigge
 * Modified GameView to have a vector of PlayModes. Is not currently used, yet.
 *
 * Revision 1.10  2001/04/20 12:28:39  sigge
 * Restructured include files. Now uses the decision tree.
 *
 * Revision 1.9  2001/04/18 14:43:37  mikbr212
 * uses passtree
 *
 * Revision 1.8  2001/04/15 00:17:46  andbo928
 * Now uses the new serialize() and deserialize() methods.
 *
 * Revision 1.7  2001/04/13 18:17:22  andbo928
 * Removed old skills.
 *
 * Revision 1.6  2001/04/13 16:30:40  sigge
 * Removed some spam.
 *
 * Revision 1.5  2001/04/13 16:28:09  sigge
 * Added communication
 *
 * Revision 1.4  2001/04/13 11:17:44  mikbr212
 * changed display
 *
 * Revision 1.3  2001/04/12 00:16:08  andbo928
 * Simplified to aid debugging.
 *
 * Revision 1.2  2001/04/08 13:25:05  sigge
 * Removed most of the old tests and updated .cvsignore
 *
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.1  2000/11/01 10:40:53  andbo928
 * Initial revision
 *
 * Revision 1.2  2000/09/05 21:12:55  frehe
 * Made FCFoo compile with RoboSoc 2.5.0
 *
 * Revision 1.1.1.1  2000/08/31 13:26:05  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FCFooDecision.cc
 * Implementation of the class FCFooDecision.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.29 $
 * $Author: mikbr212 $
 * $Date: 2001/07/06 10:01:49 $
 */

#include "FCFooDecision.h"

// System headers
#include <unistd.h>
#include <errno.h>
#include <string>

// Views
#include "view_constants.h"
#include "PassTreeView.h"

// Skills
#include "CatchBall.h"
#include "DashToSpeedVector.h"
#include "Dribble.h"
#include "FindBall.h"
#include "FuzzyClearBall.h"
#include "FuzzyScore.h"
#include "FuzzyTeleport.h"
//  #include "Go.h"
#include "Intercept.h"
#include "MarkPosition.h"
#include "MoveToPosition.h"
#include "Pass.h"
#include "PlanContainer.h"
#include "Wait.h"

// Predicates
#include "BallKickedToMe.h"
#include "IAmClosestToBall.h"
#include "InPenaltyArea.h"
#include "PlayModePredicate.h"
#include "WeHaveBall.h"

// Framework
#include "ActuatorInterface.h"
#include "rs_assert.h"
#include "rs_log.h"

// Commands
#include "ChangeViewCommand.h"
#include "DashCommand.h"
#include "InitCommand.h"
#include "KickCommand.h"
#include "MoveCommand.h"
#include "SayCommand.h"
#include "TurnNeckCommand.h"
#include "TurnCommand.h"

// Misc
#include "LoadConfig.h"


RS_BEGIN_NAMESPACE

// We love global variables.

map<string,TVector> formations;

FCFooDecision::FCFooDecision(ActuatorInterface* const actuator_interface)
  : Decision(actuator_interface),
    viewManager(ViewManager::instance()),
    parameterView(NULL),
    gameView(NULL),
    markerView(NULL),
    communicationView(NULL),
    worldStateView(NULL),
    lookView(NULL),
    passTreeView(NULL),
    serverAlive(true),
    busy(false)
#ifdef RS_USE_DISPLAY
  , theDisplay(Display::instance())
#endif
#ifdef RS_USE_COACH_DEBUGGING
  , coachView(NULL)
#endif
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
  getView(markerView, MARKER_VIEW_ID, parameterView);
  getView(communicationView, COMMUNICATION_VIEW_ID);
  getView(worldStateView, WORLD_STATE_VIEW_ID);
  getView(lookView, LOOK_VIEW_ID);

  getView(passTreeView, PASS_TREE_VIEW_ID);

#ifdef RS_USE_COACH_DEBUGGING
  getView(coachView, COACH_VIEW_ID);
#endif
}


FCFooDecision::~FCFooDecision()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
  viewManager->releaseView(MARKER_VIEW_ID);
  viewManager->releaseView(COMMUNICATION_VIEW_ID);
  viewManager->releaseView(WORLD_STATE_VIEW_ID);
  viewManager->releaseView(LOOK_VIEW_ID);
#ifdef RS_USE_COACH_DEBUGGING
  viewManager->releaseView(COACH_VIEW_ID);
#endif
}


ServerCommand* FCFooDecision::initAgent()
{
  RS_REQUIRE( parameterView != NULL );
  InitCommand* init_cmd = new InitCommand(parameterView->CP_team_name, 5.5,
					  parameterView->CP_goalie);
  return init_cmd;
}


bool FCFooDecision::start()
{
  RS_REQUIRE( initialized == true );
  while ( serverAlive )
  {
    if ( pause() == -1 && errno != EINTR )
      RS_ERROR("Something went wrong in pause!");
//     decide();
  }
  return true;
}

//  Vector FCFooDecision::getGoalVector() {
//    Vector kickVec(0,1);
//    VectorUCO toTheirGoal = VectorUCO(agentView->getAgentPosition(NOW),
//  				    markerView->getTheirGoal(NOW).getPosition());
//    toTheirGoal += Vector(90,3);
//    if(toTheirGoal.isKnown()) {
//      kickVec = toTheirGoal.getVector();
//      if(agentView->getBodyDirection(NOW).isKnown())
//        kickVec.rotate(- agentView->getBodyDirection(NOW).getDeg());
//    }
//    //  kickVec.setLength(0.5);
//    kickVec.setLength(5);
//    return kickVec;
//  }

Skill* FCFooDecision::chooseSkill() {
  static FindBall findBall;
//    static Go go; 
  static FuzzyScore score;
  static IAmClosestToBall iAmClosestToBall;
  static InPenaltyArea inPenaltyArea;
  static FuzzyClearBall clearBall;
  static Pass pass;
  static Dribble dribble;
  static BallKickedToMe ballKickedToMe(30.0, 10);
  static FuzzyTeleport teleport;
  static PlanContainer planContainer;
  static CatchBall catchBall;
  static MarkPosition markPosition;
  static MoveToPosition moveToPosition;
  static Intercept intercept;
  static Wait wait;

  const AgentState& agent(worldStateView->getWorldState().getAgent());
  const BallState& ball(worldStateView->getWorldState().getBall());

  vector<PlayMode> playModes = gameView->getPlayModes(NOW);
  if(find(playModes.begin(), playModes.end(), PM_BEFORE_KICK_OFF) != playModes.end()
     || find(playModes.begin(), playModes.end(), PM_HALF_TIME) != playModes.end()
     || find(playModes.begin(), playModes.end(), PM_THEIR_GOAL) != playModes.end())
  {
    TVector position = formations["init"];
    Point p(position.getX(), position.getY());
    MoveCommand move_cmd(p);
    Step s(&move_cmd);
    planContainer.setPlan(Plan(s));
    return &planContainer;
  }
  else if(find(playModes.begin(), playModes.end(), PM_OUR_GOAL) != playModes.end())
  {
    TVector position = formations["init-defense"];
    Point p(position.getX(), position.getY());
    MoveCommand move_cmd(p);
    Step s(&move_cmd);
    planContainer.setPlan(Plan(s));
    return &planContainer;
  }

  if(parameterView->CP_goalie)
  {
    // ------
    // Goalie
    // ------
    if(teleport.applicable()) {
      return &teleport;
    }

    if(catchBall.applicable() > FuzzyBool(0.3)) {
      return &catchBall;
    }

    if(clearBall.applicable()) {
      return &clearBall;
    }

    const vector<PlayerState>& opps = 
      worldStateView->getWorldState().getOpponents();
    int minTime = 1000;
    for(int i = 0; i < opps.size(); i++)
    {
      int curTime = opps[i].getInterceptTime(ball);
      if(curTime != -1
	 && curTime < minTime)
      {
	minTime = curTime;
      }
    }
    intercept.setMaxSteps(min(minTime+1, 10));
    if(intercept.applicable()) {
      return &intercept;
    }

    if(findBall.applicable(0.2)) {
      return &findBall;
    }

    // calculate goalie-position
    markPosition.setLookAtMark(false);
    TVector goal = TVector(-52.5, 0);
    float angle = abs((ball.getPosition() - goal).getAngle());
    float distance = 7.0 - 2.0*(angle/90.0);
    float xpos = -52.5 + 5.0*(angle/90.0);
    float ypos = 0;
//     float ypos = ball.getPosition().getY() / parameterView->SP_pitch_width
//       * parameterView->SP_goal_width;
    TVector markPos = TVector(xpos, ypos);
    TVector markDir = distance * (ball.getPosition() - markPos).normalized();
    Display::instance()->addArrow(markPos, markPos + markDir);
    markPosition.setDistance(distance);
    markPosition.setPosition(markPos);
    return &markPosition;

  } else {
    // ------------------
    // Non-goalie players
    // ------------------

    if(findBall.applicable(0.1))
      return &findBall;

    // IHaveBall?
    if(agent.isBallKickable(ball))
    {
      if(wait.applicable())
	return &wait;

      // Ball is kickable
      double scoreApp = score.applicable().getValue();
      double passApp = pass.applicable().getValue();
      cout << "passapp: " << passApp << endl;

      if(scoreApp > 0.5)
	return &score;

      if(passApp > 0.4)
	return &pass;

      if(scoreApp > 0.3)
	return &score;

      const vector<PlayerState>& opponents = worldStateView->getWorldState().getOpponents();
      float closest = 10000000.0;
      for(unsigned int i=0; i<opponents.size(); i++)
      {
	float distance = (opponents[i].getPosition() - agent.getPosition()).getLength();
	if(distance < closest)
	  closest = distance;
      }
      if(closest > 5.0)
      {
	dribble.setPosition(markerView->getTheirGoal(NOW).getPosition());
	return &dribble;
      }

      if(passApp > 0.2)
	return &pass;

      return &dribble;
    } else {
      // Ball is _not_ kickable
      return &dribble; // &go;
    }
  }
}

int lastCycle = -1;
bool FCFooDecision::decide()
{
  if(busy)
    return true;
  busy = true;
  int gameTime = worldStateView->getWorldState().getGameTime();
  if(lastCycle == gameTime)
  {
    busy = false;
    return true;
  }
  lastCycle = gameTime;

//   cout << "bodyDir=" << agent.getDirection() << endl;
//   Float f;
//   AngleDeg a = lookView->getAngle(f, agent.getDirection()-90, agent.getDirection()+90);
//   cout << "a=" << a.getDeg() << ", f=" << f << endl;

  static Command* cmd = NULL;
  static Plan plan = Plan(Step(NULL));
  static Skill* chosenSkill = NULL;
  static Command* noCmd = new TurnCommand(0);
  
  const AgentState& agent(worldStateView->getWorldState().getAgent());

  if(communicationView->isThisMySpeakCycle(worldStateView->getWorldState()))
  {
    const WorldState& curWorld = worldStateView->getWorldState();
    string finstreng = worldStateView->getWorldState().serialize();
      //      cout << "Message-pre64-length: " << finstreng.length() << endl;
//        WorldState testWorld;
//        testWorld.deserialize(finstreng);
      
//        cout << "==================" << endl
//  	   << "PRE-SERIALIZATION" << endl
//  	   << "==================" << endl
//  	   << "Time: " << curWorld.getGameTime() << endl
//  	   << "Teammates: " << curWorld.getTeammates().size() << endl
//  	   << "Opponents: " << curWorld.getOpponents().size() << endl
//  	   << "Ball: ============" << endl
//  	   << "Pos:" << curWorld.getBall().getPosition() << endl
//  	   << "Spd:" << curWorld.getBall().getSpeed() << endl
//  	   << "Agent:  ==========" << endl
//  	   << "Stm: " << curWorld.getAgent().getStamina() << " "
//  	   << "Rec: " << curWorld.getAgent().getRecovery() << " "
//  	   << "Eff: " << curWorld.getAgent().getEffort() << " " << endl;
//        printPlayer(curWorld.getAgent());
//        const vector<PlayerState>& tm = curWorld.getTeammates();
//        for(int i = 0; i < tm.size(); i++)
//  	{
//  	  printPlayer(tm[i]);
//  	}

//        cout << "==================" << endl
//  	   << "POST-SERIALIZATION" << endl
//  	   << "==================" << endl
//  	   << "Time: " << testWorld.getGameTime() << endl
//  	   << "Teammates: " << testWorld.getTeammates().size() << endl
//  	   << "Opponents: " << testWorld.getOpponents().size() << endl
//  	   << "Ball: ============" << endl
//  	   << "Pos:" << testWorld.getBall().getPosition() << endl
//  	   << "Spd:" << testWorld.getBall().getSpeed() << endl
//  	   << "Agent:  ==========" << endl
//  	   << "Stm: " << testWorld.getAgent().getStamina() << " "
//  	   << "Rec: " << testWorld.getAgent().getRecovery() << " "
//  	   << "Eff: " << testWorld.getAgent().getEffort() << " " << endl;
//        printPlayer(testWorld.getAgent());
//        tm = testWorld.getTeammates();
//        for(int i = 0; i < tm.size(); i++)
//  	{
//  	  printPlayer(tm[i]);
//  	}

    communicationView->addMessage(Message(worldStateMessageType, finstreng));
      //      cout << "Fin2: " << finstreng << endl;

    string message = communicationView->getOutgoing();
      //      cout << "message.size()=" << message.size() << endl;
    if(message.size() > 510)
      cout << "Too large message!" << endl;
    else
    {
      SayCommand sayCommand(message);
      actuatorInterface->addToPlan(&sayCommand);
    }
  }

  if(plan.empty()) {

    // check persistence
    if(!(chosenSkill != NULL &&
	 chosenSkill->persistent() &&
	 chosenSkill->applicable()))
      chosenSkill = chooseSkill();

    // apply skill
    if(chosenSkill != NULL) 
      plan = chosenSkill->generatePlan();

  } // end if(plan.empty())
  
  
  if(!plan.empty()) {
    Step step = plan.getNextStep();
    plan.removeNextStep();
    while(!step.empty()) {
      cmd = step.getNextCommand();
      step.removeNextCommand();
      if ( cmd != NULL ) {
	actuatorInterface->addToPlan(cmd);
	delete cmd;
      }
    } 
  } else {
    actuatorInterface->addToPlan(noCmd); // avoid spam
  }

  busy = false;
  return true;
  
}


void FCFooDecision::onActuatorSensorData()
{
#ifdef RS_USE_DISPLAY
//    theDisplay->setLastSkill(commandView->getLastServerString());
#endif
#ifdef RS_USE_OWN_LOGFILE
//    logFile << gameView->getCurrentTime() << ": executed action "
//  	  << commandView->getLastServerString()
//  	  << std::endl;
//    if ( commandView->getActualTurn(NOW) > AngleDeg(0) )
//      logFile << gameView->getCurrentTime() << ": actual turn = "
//  	    << commandView->getActualTurn(NOW)
//  	    << std::endl;  
#endif
}


void FCFooDecision::onInit()
{
  // Let the goalie use narrow vision to get rapid updates
  if(parameterView->CP_goalie)
  {
    ChangeViewCommand cvc(VW_NARROW, VQ_HIGH);
    actuatorInterface->addToPlan(&cvc);
  }

  const AgentState& agent(worldStateView->getWorldState().getAgent());
  
  UInt16 no = agent.getUniformNumber();
  if(no < 1 || no > 11)
    no = 1;

  formations = LoadConfig("fcfoo.conf")[no];
  TVector position = formations["init"];
  Point p(position.getX(), position.getY());
  MoveCommand move_cmd(p);
  actuatorInterface->addToPlan(&move_cmd);
}


void FCFooDecision::onVisualSensorData()
{
  //RS_LOG( LA_DECISION, gameView->getCurrentTime() << ": **saw**" );
}


void FCFooDecision::onAfterSensorData()
{
#ifdef RS_USE_DISPLAY
  theDisplay->draw();
#endif
}


void FCFooDecision::onNewCycle(const bool forced)
{
  /*  if ( forced )
      RS_LOG( LA_DECISION,
      gameView->getCurrentTime() << ": **Forced NewCycle**" );
      else
      RS_LOG( LA_DECISION, gameView->getCurrentTime() << ": **NewCycle**" );
  */
}


bool FCFooDecision::onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks)
{
  // We assume that if the sense_body is more than four ticks late
  // then it's lost.
  if ( false )
    std::cout << "onEstimatedNewCycle(" << estimated_time
	      << ", " << ticks << ")" << endl;

  // If more than nine sense_body's are missed in a row then assume that
  // the server is dead.
  if ( estimated_time > 9 )
    serverAlive = false;

  if ( ticks > 4 ) {
    // TODO: change this into sending a control command instead!
    //sensorInterface->ForceNewCycle();
    return true;
  } else
    return false;
}


void FCFooDecision::onCommandWarning()
{
  decide();
}


RS_END_NAMESPACE

