/* -*- Mode: C++ -*-
 * $Id: Mickes_FCFooDecision.cc,v 1.1.1.1 2001/04/08 11:33:59 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FCFooDecision.cc
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.1.1.1 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: sigge $ $Date: 2001/04/08 11:33:59 $
 *
 *
 * HISTORY
 *
 * $Log: Mickes_FCFooDecision.cc,v $
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.2  2000/09/05 21:12:55  frehe
 * Made FCFoo compile with RoboSoc 2.5.0
 *
 * Revision 1.1.1.1  2000/08/31 13:26:05  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FCFooDecision.cc
 * Implementation of the class FCFooDecision.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:59 $
 */


#include "FCFooDecision.h"

#include <unistd.h>
#include <errno.h>

#include "view_constants.h"
#include "ActuatorInterface.h"
#include "InitCommand.h"
#include "MoveCommand.h"
#include "TurnCommand.h"
#include "KickCommand.h"
#include "TurnNeckCommand.h"
#include "DashCommand.h"

#include "FindBall.h"
#include "GotoBall.h"
#include "Score.h"
#include "KickBallToSpeedVector.h"
#include "KickBallToSpeedVectorMulti.h"
#include "TurnBall.h"
#include "InterceptBallStamina.h"
#include "DribbleToPosition.h"
#include "ZoneDefence.h"
#include "CatchBall.h"
#include "CutOffGoalAngleStamina.h"

#include "IAmClosestToBall.h"
#include "BallKickedToMe.h"
#include "IAmGoalie.h"

#include "rs_assert.h"
#include "rs_log.h"


RS_BEGIN_NAMESPACE

FCFooDecision::FCFooDecision(ActuatorInterface* const actuator_interface)
  : Decision(actuator_interface),
    viewManager(ViewManager::instance()),
    parameterView(NULL),
    gameView(NULL),
    commandView(NULL),
    ballView(NULL),
    agentView(NULL),
    playerView(NULL),
    kickBallView(NULL),
    markerView(NULL),
    serverAlive(true)
#ifdef RS_USE_DISPLAY
  , theDisplay()
#endif
#ifdef RS_USE_COACH_DEBUGGING
  , coachView(NULL)
#endif
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(gameView, GAME_VIEW_ID);
  getView(commandView, COMMAND_VIEW_ID);
  //markerView = RS_GET_VIEW(CMarkerView, MARKER_VIEW_ID);
  getView(ballView, BALL_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(playerView, PLAYER_VIEW_ID);
  getView(kickBallView, KICKBALL_VIEW_ID, parameterView,
	  ballView, agentView, commandView);
  getView(markerView, MARKER_VIEW_ID, parameterView);
#ifdef RS_USE_COACH_DEBUGGING
  getView(coachView, COACH_VIEW_ID);
#endif
}


FCFooDecision::~FCFooDecision()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(COMMAND_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
  viewManager->releaseView(KICKBALL_VIEW_ID);
  viewManager->releaseView(MARKER_VIEW_ID);

#ifdef RS_USE_COACH_DEBUGGING
  viewManager->releaseView(COACH_VIEW_ID);
#endif
}


ServerCommand* FCFooDecision::initAgent()
{
  RS_REQUIRE( parameterView != NULL );
  InitCommand* init_cmd = new InitCommand(parameterView->CP_team_name, 5.5,
					  parameterView->CP_goalie);
  return init_cmd;
}


bool FCFooDecision::start()
{
  RS_REQUIRE( initialized == true );
  while ( serverAlive )
    if ( pause() == -1 && errno != EINTR )
      RS_ERROR("Something went wrong in pause!");
  return true;
}

Vector FCFooDecision::getGoalVector() {
  Vector kickVec(0,1);
  VectorUCO toTheirGoal = VectorUCO(agentView->getAgentPosition(NOW),
				    markerView->getTheirGoal(NOW).getPosition());
  if(toTheirGoal.isKnown()) {
    kickVec = toTheirGoal.getVector();
    if(agentView->getBodyDirection(NOW).isKnown())
      kickVec.rotate(- agentView->getBodyDirection(NOW).getDeg());
  }
  kickVec.setLength(2.3);
  return kickVec;
}

Skill* FCFooDecision::chooseSkill() {
  static InterceptBallStamina interceptBall;
  static FindBall findBall;
  static GotoBall gotoBall;
  static Score score;
  static KickBallToSpeedVector kickball(Vector(0,0));
  static TurnBall turnball(Vector(135.0,0.7)); 
  static KickBallToSpeedVectorMulti kickballM(Vector(-90.0,2.3), true, false);
  static PlanContainer planContainer;
  static IAmClosestToBall iAmClosestToBall;
  static IAmGoalie iAmGoalie;
  static BallKickedToMe ballKickedToMe;
  static DribbleToPosition dribble(Point(40,0));
  static ZoneDefence zoneDefence(Zone(-10.0, 0.0, 5.0));
  static CatchBall catchBall;
  static CutOffGoalAngleStamina cutOffGoalAngle;

  static float r = 5.0;
  static Zone zones[] = { Zone(-50, 0, r),

			   Zone(-32.5, 16, r),
			   Zone(-32.5, 0,  r),
			   Zone(-32.5,-16, r),
			   
			   Zone(-22.5, 24, r),
			   Zone(-22.5, 8,  r),
			   Zone(-22.5,-8,  r),
			   Zone(-22.5,-24, r),
			   
			   Zone(-12.5, 16, r),
			   Zone(-12.5, 0,  r),
			   Zone(-12.5,-16, r)
  };
  
  if(agentView->getUniformNumber().isKnown())
    zoneDefence.setZone(zones[agentView->getUniformNumber().getValue() - 1]);

  //  kickballM.setFastKick(true,false);
  /*  turnball.setPlayerMargin(15.0);
      turnball.setKickableMargin(60.0);
      turnball.setAccuracy(0.2);
      turnball.setMaxAcceleration(0.1); */

  /*
    static FuzzyBool turnRight=TRUE;
    if(!turnball.applicable(Vector(90.0, 0.6)) && turnRight)
    turnRight=FALSE;
    
    if(!turnball.applicable(Vector(-30.0, 0.6)) && !turnRight)
    turnRight=TRUE;
    
    
    if(turnball.applicable(Vector(90.0, 0.6)) && turnRight) {
    cout << "Turn right: ";
    plan = turnball.generatePlan(Vector(90.0, 0.6));
    cout << endl;
    } else if(turnball.applicable(Vector(-30.0, 0.6))) {
    cout << "Turn left: ";
    plan = turnball.generatePlan(Vector(-30.0, 0.6));
    cout << endl;
    } else
  */
  /*
    if(kickball.applicable(Vector(90.0,1.0))) {
    plan = kickball.generatePlan(Vector(90.0,1.0));
    } else
  */

  if(iAmGoalie()) {
    if(catchBall.applicable()) {
      /* Catch ball */
      cout << "Catch ball" << endl;
      return &catchBall;
    } else if(cutOffGoalAngle.applicable()) {
      /* Cut of goal angle */
      cout << "Cut of goal angle" << endl;
      return &cutOffGoalAngle;
    } 
  } else { // not goalie
      
    if(false && playerView->getClosestTeammate(NOW) != NULL &&
       playerView->getClosestTeammate(NOW)->getPosition().isKnown() &&
       agentView->getAgentPosition(NOW).isKnown()) {
      Vector kickVec = Vector(agentView->getAgentPosition(NOW).getPoint(),
			      playerView->getClosestTeammate(NOW)->getPosition().getPoint());
      /*    cout << "Angle = " << kickVec.getAngle().getDeg() 
	    << ", distance = " << kickVec.getLength() << endl; */
      
      kickVec /= (1 - pow(parameterView->SP_ball_decay,20)) /
	(1 - parameterView->SP_ball_decay);
      kickballM.setVector(kickVec);
    } else {
      kickballM.setVector(getGoalVector());
      //    cout << "Can't find closest player\n";
    }
    
    //  if(score.applicable()) {
    /* Score */
    //    return &score;
    //  } else if(dribble.applicable() && iAmClosestToBall()) {
    /* Dribble */
    //    return &dribble;
    //} else 
    if (kickballM.applicable() && iAmClosestToBall()) {
      /* Multikick */
      return &kickballM;
      cout << "Multikick" << endl;
    } else if(ballKickedToMe() && interceptBall.applicable()) {
      /* Intercept ball */
      return &interceptBall;
      cout << "Intercept ball" << endl;
    } else if (gotoBall.applicable() && iAmClosestToBall()) {
      /* GotoBall */
      return &gotoBall;
      cout << "GotoBall" << endl;
    } else if ( agentView->getNeckDirection(NOW).isKnown() &&
		ballView->getRelativeBallDirection(NOW).isKnown()) {
      //	      abs(agentView->getNeckDirection(NOW).getDeg()) > 10 ) {
      /* Turn (look at ball) */
      Step step(new TurnNeckCommand(-agentView->getNeckDirection(NOW).getDeg()));
      step.addCommand(new TurnCommand(ballView->getRelativeBallDirection(NOW).getDeg() 
				      + agentView->getNeckDirection(NOW).getDeg()));
      planContainer.setPlan(Plan(step)); 
      return &planContainer;
    } else if(zoneDefence.applicable()) {
      cout << "Zone Defence" << endl;
      /* Zone Defence */
      return &zoneDefence;
    }
  }
  
  // Common skills
  if ( findBall.applicable() ) {
    /* FindBall */
    cout << "Findball" << endl;
    return &findBall;
  } if(!agentView->getAgentPosition(NOW).isKnown()) {
    planContainer.setPlan(Step(new TurnCommand(45)));
    /* Where am I? */
    cout << "Where am I?" << endl;
    return &planContainer;
  } else {
    cout << "Empty plan" << endl;
    /* Empty plan */
    return NULL;
  }
}

bool FCFooDecision::decide()
{
  static Command* cmd = NULL;
  static Plan plan = Plan(Step(NULL));
  static Skill* lastSkill = NULL;
  static Command* noCmd = new TurnCommand(0);

  if(plan.empty()) {
    if(lastSkill == NULL ||
       !lastSkill->persistent() || !lastSkill->applicable())
      lastSkill = chooseSkill();

    // apply skill
    if(lastSkill != NULL) 
      plan = lastSkill->generatePlan();
    
    
  } // end if(plan.empty())
  
  
  if(!plan.empty()) {
    Step step = plan.getNextStep();
    plan.removeNextStep();
    while(!step.empty()) {
      cmd = step.getNextCommand();
      step.removeNextCommand();
      if ( cmd != NULL ) {
	actuatorInterface->addToPlan(cmd);
	delete cmd;
      }
    } //while
  } else {
    actuatorInterface->addToPlan(noCmd); // avoid spam
  }

  // To test the quality of the information available to the
  // agent compare the estimated values with the correct values
#ifdef RS_USE_OWN_LOGFILE
  static Float ballXError = 0;
  static Float ballYError = 0;
  static Float ballDirError = 0;
  static Float ballDistError = 0;
  static Float agentXError = 0;
  static Float agentYError = 0;
  static Float agentFaceError = 0;
  static Float agentBodyError = 0;
  static Float agentNeckError = 0;
  static Float agentFaceMaxError = 0;
  static Float agentBodyMaxError = 0;
  static Float agentNeckMaxError = 0;
    
  if ( coachView->getCoachMessagetime(NOW) != gameView->getCurrentTime().getT() ) {
    logFile << "Coach time and current time not equal! "
	    << coachView->getCoachMessagetime(NOW)
	    << " != " << gameView->getCurrentTime().getT() << std::endl;
  } else {
    FloatUnknown ballX = ballView->getBallPosition(NOW).getX();
    FloatUnknown ballY = ballView->getBallPosition(NOW).getY();
    FloatUnknown ballRealX = coachView->getBallAbsolutePosition(NOW).getX();
    FloatUnknown ballRealY = coachView->getBallAbsolutePosition(NOW).getY();
    FloatUnknown ballXDiff = abs(ballX - ballRealX);
    FloatUnknown ballYDiff = abs(ballY - ballRealY);
    AngleDegUnknown ballSpeedDir = ballView->getBallAbsoluteSpeedVector(NOW).getAngle();
    FloatUnknown ballSpeed = ballView->getBallAbsoluteSpeedVector(NOW).getLength();
    AngleDegUnknown ballRealSpeedDir = coachView->getBallAbsoluteSpeed(NOW).getAngle();
    FloatUnknown ballRealSpeed = coachView->getBallAbsoluteSpeed(NOW).getLength();

    if ( ballXDiff.isKnown() )
      ballXError += ballXDiff.getValue();
    if ( ballYDiff.isKnown() )
      ballYError += ballYDiff.getValue();
    

    AngleDegUnknown ballDir = ballView->getRelativeBallDirection(NOW);
    FloatUnknown ballDist = ballView->getBallDistance(NOW);
    AngleDegUnknown ballRealDir = coachView->getRelativeBallVector(NOW).getAngle();
    FloatUnknown ballRealDist = coachView->getRelativeBallVector(NOW).getLength();
    AngleDegUnknown ballDirDiff = abs(ballDir - ballRealDir);
    FloatUnknown ballDistDiff = abs(ballDist - ballRealDist);
    
    if ( ballDirDiff.isKnown() )
      ballDirError += ballDirDiff.getDeg();
    if ( ballDistDiff.isKnown() )
      ballDistError += ballDistDiff.getValue();
    

    FloatUnknown agentX = agentView->getAgentPosition(NOW).getX();
    FloatUnknown agentY = agentView->getAgentPosition(NOW).getY();
    FloatUnknown agentRealX = coachView->getAgentAbsolutePosition(NOW).getX();
    FloatUnknown agentRealY = coachView->getAgentAbsolutePosition(NOW).getY();
    FloatUnknown agentXDiff = abs(agentX - agentRealX);
    FloatUnknown agentYDiff = abs(agentY - agentRealY);
    AngleDegUnknown agentDir = agentView->getSpeedVector(NOW).getAngle();
    FloatUnknown agentSpeed = agentView->getSpeedVector(NOW).getLength();
    AngleDegUnknown agentRealDir = coachView->getAgentAbsoluteSpeed(NOW).getAngle();
    FloatUnknown agentRealSpeed = coachView->getAgentAbsoluteSpeed(NOW).getLength();

    if ( agentXDiff.isKnown() )
      agentXError += agentXDiff.getValue();
    if ( agentYDiff.isKnown() )
      agentYError += agentYDiff.getValue();
    
    AngleDegUnknown agentBody = agentView->getBodyDirection(NOW);
    AngleDegUnknown agentFace = agentView->getFaceDirection(NOW);
    AngleDegUnknown agentNeck = agentView->getNeckDirection(NOW);
    AngleDegUnknown agentRealBody = coachView->getAgentAbsoluteDirection(NOW);
    AngleDegUnknown agentRealFace = coachView->getAgentAbsoluteFaceDirection(NOW);
    AngleDegUnknown agentRealNeck = coachView->getAgentAbsoluteNeckDirection(NOW);
    AngleDegUnknown agentBodyDiff = abs(agentBody - agentRealBody);
    AngleDegUnknown agentFaceDiff = abs(agentFace - agentRealFace);
    AngleDegUnknown agentNeckDiff = abs(agentNeck - agentRealNeck);

    if ( agentBodyDiff.isKnown() ) {
      agentBodyError += agentBodyDiff.getDeg();
      agentBodyMaxError = max(agentBodyDiff.getDeg(), agentBodyMaxError);
    }
    if ( agentFaceDiff.isKnown() ) {
      agentFaceError += agentFaceDiff.getDeg();
      agentFaceMaxError = max(agentFaceDiff.getDeg(), agentFaceMaxError);
    }
    if ( agentNeckDiff.isKnown() ) {
      agentNeckError += agentNeckDiff.getDeg();
      agentNeckMaxError = max(agentNeckDiff.getDeg(), agentNeckMaxError);
    }
    
    logFile << gameView->getCurrentTime() << ": ballPos"
      // << "= ("
      //    << ballX << ", " << ballY << "); ballRealPos = ("
      //    << ballRealX << ", " << ballRealY << "); "
	    << " Diff = (" << ballXDiff << ", " << ballYDiff << ")"
	    << " Sum error = (" << ballXError << ", " << ballYError << ")"
	    << std::endl;
    logFile << gameView->getCurrentTime() << ": ballSpeed"
	    << " = (" << ballSpeedDir << ", " << ballSpeed << "); ballRealSpeed = ("
	    << ballRealSpeedDir << ", " << ballRealSpeed << "); "
	    << " Diff = ("
	    << abs(ballSpeedDir - ballRealSpeedDir) << ", " << abs(ballSpeed - ballRealSpeed) << ")"
	    << std::endl;

    logFile << gameView->getCurrentTime() << ": ballVector"
	    << " = (" << ballDir << ", " << ballDist << ");"
	    << " ballRealVector = (" << ballRealDir << ", " << ballRealDist << ");"
	    << " Diff = (" << ballDirDiff << ", " << ballDistDiff << ");"
	    << " Sum error = (" << ballDirError << ", " << ballDistError << ");"
	    << std::endl;

    logFile << gameView->getCurrentTime() << ": agentPos"
      // << "= ("
      //   << agentX << ", " << agentY << "); agentRealPos = ("
      //    << agentRealX << ", " << agentRealY << "); "
	    << " Diff = (" << agentXDiff << ", " << agentYDiff << ")"
	    << " Sum error = (" << agentXError << ", " << agentYError << ")"
	    << std::endl;
    logFile << gameView->getCurrentTime() << ": agentSpeed"
      //	    << "= ("
      //    << agentDir << ", " << agentSpeed << "); agentRealSpeed = ("
      //    << agentRealDir << ", " << agentRealSpeed << "); "
	    << " Diff = ("
	    << abs(agentDir - agentRealDir) << ", " << abs(agentSpeed - agentRealSpeed) << ")"
	    << std::endl;

    logFile << gameView->getCurrentTime() << ": bodyDir"
	    << "= " << agentBody << " realBodyDir = " << agentRealBody << ";"
	    << " Diff = " << agentBodyDiff
	    << " Sum error = " << agentBodyError
      	    << " Max error = " << agentBodyMaxError 
 	    << std::endl;
    
    logFile << gameView->getCurrentTime() << ": faceDir"
	    << "= " << agentFace << " realFaceDir = " << agentRealFace << ";"
	    << " Diff = " << agentFaceDiff
	    << " Sum error = " << agentFaceError 
      	    << " Max error = " << agentFaceMaxError
 	    << std::endl;
    
    logFile << gameView->getCurrentTime() << ": neckDir"
	    << "= " << agentNeck << " realNeckDir = " << agentRealNeck << ";"
	    << " Diff = " << agentNeckDiff
	    << " Sum error = " << agentNeckError 
	    << " Max error = " << agentNeckMaxError
	    << std::endl;
  }
#endif

  return true;
  
}


void FCFooDecision::onActuatorSensorData()
{
#ifdef RS_USE_DISPLAY
  theDisplay.setLastSkill(commandView->getLastServerString());
#endif
#ifdef RS_USE_OWN_LOGFILE
  logFile << gameView->getCurrentTime() << ": executed action "
	  << commandView->getLastServerString()
	  << std::endl;
  if ( commandView->getActualTurn(NOW) > AngleDeg(0) )
    logFile << gameView->getCurrentTime() << ": actual turn = "
	    << commandView->getActualTurn(NOW)
	    << std::endl;  
#endif
}


void FCFooDecision::onInit()
{
  static Point startingPositions[] = { Point(-50,0),
				       
				       Point(-32.5, 16),
				       Point(-32.5, 0),
				       Point(-32.5,-16),
				       
				       Point(-22.5, 24),
				       Point(-22.5, 8),
				       Point(-22.5,-8),
				       Point(-22.5,-24),
				       
				       Point(-12.5, 16),
				       Point(-12.5, 0),
				       Point(-12.5,-16)
  };
  
  UInt16 no = 1;
  if(agentView->getUniformNumber().isKnown())
    no = agentView->getUniformNumber().getValue();

  MoveCommand* move_cmd = new MoveCommand(startingPositions[no - 1]);
  actuatorInterface->addToPlan(move_cmd);
}


void FCFooDecision::onVisualSensorData()
{
  //RS_LOG( LA_DECISION, gameView->getCurrentTime() << ": **saw**" );
}


void FCFooDecision::onAfterSensorData()
{
#ifdef RS_USE_DISPLAY
  theDisplay.draw();
#endif
}


void FCFooDecision::onNewCycle(const bool) // forced)
{
  /*  if ( forced )
      RS_LOG( LA_DECISION,
      gameView->getCurrentTime() << ": **Forced NewCycle**" );
      else
      RS_LOG( LA_DECISION, gameView->getCurrentTime() << ": **NewCycle**" );
  */
}


bool FCFooDecision::onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks)
{
  // We assume that if the sense_body is more than four ticks late
  // then it's lost.
  if ( false )
    std::cout << "onEstimatedNewCycle(" << estimated_time
	      << ", " << ticks << ")" << endl;

  // If more than nine sense_body's are missed in a row then assume that
  // the server is dead.
  if ( estimated_time > 9 )
    serverAlive = false;

  if ( ticks > 4 ) {
    // TODO: change this into sending a control command instead!
    //sensorInterface->ForceNewCycle();
    return true;
  } else
    return false;
}


void FCFooDecision::onCommandWarning()
{
  decide();
}


RS_END_NAMESPACE

