/* -*- Mode: C++ -*-
 * $Id: Display.cc,v 1.12 2001/08/01 14:17:05 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Display.cc
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.12 $
 * Created by: Johan Thapper 1999-xx-xx
 * Last modified by $Author: mikbr212 $ $Date: 2001/08/01 14:17:05 $
 *
 *
 * HISTORY
 *
 * $Log: Display.cc,v $
 * Revision 1.12  2001/08/01 14:17:05  mikbr212
 * Removed debug spam
 *
 * Revision 1.11  2001/07/11 16:35:50  sigge
 * Modified Monitor protocol for size and decoding speed.
 *
 * Revision 1.10  2001/07/04 13:21:36  sigge
 * Added role display.
 *
 * Revision 1.9  2001/04/24 17:44:59  sigge
 * Check for file presence before opening it.
 *
 * Revision 1.8  2001/04/20 14:46:38  sigge
 * Now uses the username in the fifo names, to avoid conflicts on shared machines (astmatix). Also added addArrow with TLine
 *
 * Revision 1.7  2001/04/19 12:27:12  mikbr212
 * Changed arrow-time to cycles (int)
 *
 * Revision 1.6  2001/04/16 14:21:49  mikbr212
 * fixed uniform numbers
 *
 * Revision 1.5  2001/04/14 23:43:10  andbo928
 * It now opens /tmp/rs_monitor<num>.txt.
 *
 * Revision 1.4  2001/04/13 14:56:59  mikbr212
 * fixed stuff
 *
 * Revision 1.3  2001/04/13 14:56:20  mikbr212
 * fixed stuff
 *
 * Revision 1.2  2001/04/13 11:18:28  mikbr212
 * Added ::instance()
 *
 * Revision 1.1  2001/04/13 10:41:29  mikbr212
 * Moved Display
 *
 * Revision 1.6  2001/04/12 17:07:05  andbo928
 * Now plots agentdir and neckdir as arrows and displays stamina correctly.
 *
 * Revision 1.5  2001/04/12 07:53:01  mikbr212
 * flushed file
 *
 * Revision 1.4  2001/04/12 00:16:07  andbo928
 * Simplified to aid debugging.
 *
 * Revision 1.3  2001/04/11 18:54:52  andbo928
 * Fixed some stuff.
 *
 * Revision 1.2  2001/04/11 16:24:35  andbo928
 * Fixed display to show a lot of thingys...
 *
 * Revision 1.2  2001/04/11 16:07:48  andbo928
 * Fixed display to show a lot of thingys...
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.4  2000/10/19 00:05:34  frehe
 * Made minor changes.
 *
 * Revision 1.3  2000/10/18 23:07:11  frehe
 * Added a say message and a display of them.
 *
 * Revision 1.2  2000/10/11 18:40:29  frehe
 * Upgraded FCFoo.
 *
 * Revision 1.1  2000/09/05 21:12:54  frehe
 * Made FCFoo compile with RoboSoc 2.5.0
 *
 */

/**
 * @file Display.cc
 * Implementation file for the class Display.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.12 $
 * $Author: mikbr212 $
 * $Date: 2001/08/01 14:17:05 $
 */

//#define USE_DISPLAY

#include <stdio.h>
#include "TVector.h"
#include "BallState.h"
#include "WorldState.h"
#include "PlayerState.h"
#include "Display.h"
#include "view_constants.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <netinet/in.h>

RS_BEGIN_NAMESPACE


Display* Display::theOneAndOnly(NULL);

Display* Display::instance() {
  if(theOneAndOnly == NULL)
    theOneAndOnly = new Display();
  return theOneAndOnly;
}

Display::Display()
  : viewManager(ViewManager::instance()),
#ifdef RS_USE_COACH_DEBUGGING
    coachView(NULL),
#endif
    worldView(NULL),
    enabled(true)
{
#ifdef RS_USE_COACH_DEBUGGING
    getView(coachView, COACH_VIEW_ID);
#endif
    getView(worldView, WORLD_STATE_VIEW_ID);
    is_open = 0;
}


Display::~Display()
{
#ifdef RS_USE_COACH_DEBUGGING
  viewManager->releaseView(COACH_VIEW_ID);
#endif
  viewManager->releaseView(WORLD_STATE_VIEW_ID);
}


void Display::setLastSkill(const std::string& last)
{
    lastSkill = last;
}


void Display::addArrow(const TLine& line,
		       int duration,
		       Display::Color color)
{
#ifdef USE_DISPLAY
  addArrow(line.getStart(), line.getEnd(), duration, color);
#endif
}

void Display::addArrow(const TVector& start,
		       const TVector& end,
		       int duration,
		       Display::Color color)
{
#ifdef USE_DISPLAY
    arrowStart.push_back(start);
    arrowEnd.push_back(end);
    arrowDuration.push_back(duration);
    arrowColor.push_back(color);
#endif
}

void Display::addCircle(const TVector center,
	       float radius,
	       int duration = 1,
	       Display::Color color)
{
#ifdef USE_DISPLAY
  circleCenters.push_back(center);
  circleRadii.push_back(radius);
  circleDuration.push_back(duration);
  circleColor.push_back(color);
#endif
}

void Display::setEnabled(bool newStatus)
{
  enabled = newStatus;
}

void Display::addMessage(string msg)
{
#ifdef USE_DISPLAY
  messages.insert(messages.end(), msg);
#endif
}

void Display::PutFloat(float f)
{
  unsigned short s = htons((int)((0.5*f + 0.5)*65535.0));
  write(fd, &s, 2);
}

void Display::PutFloatByte(float f)
{
  unsigned char c = ((0.5*f + 0.5)*255.0);
  write(fd, &c, 1);
}

void Display::PutShort(int i)
{
  unsigned short s = htons(i);
  write(fd, &s, 2);
}

void Display::PutByte(int i)
{
  unsigned char c = i;
  write(fd, &c, 1);
}

void Display::PutPlayer(const PlayerState& player, int team, int me)
{
  TVector pos = player.getPosition();
  PutFloat(pos.getX()/52.5);
  PutFloat(pos.getY()/34.0);
  PutFloatByte(pos.getC());
  int unum = player.getUniformNumber();
  int misc = team | ((unum == -1 ? 0 : unum) << 2) | (me << 6);
  PutByte(misc);
  PutFloatByte(player.getDirection());
}

void Display::draw()
{
#ifdef USE_DISPLAY

  const WorldState& ws = worldView->getWorldState();

  if(!enabled)
    {
      arrowStart.clear();
      arrowEnd.clear();
      arrowDuration.clear();
      arrowColor.clear();

      circleCenters.clear();
      circleRadii.clear();
      circleDuration.clear();
      circleColor.clear();

      messages.clear();
      return;
    }

  if(ws.getAgent().getUniformNumber() == -1)
    return;

  if(!is_open)
    {
      char buf[256];
      char* userName = getenv("USER");
      snprintf(buf, 256, "/tmp/rs_monitor%i_%s.txt",
	       ws.getAgent().getUniformNumber(), userName);
      // Check for file existance
      struct stat file_stat;
      if(stat(buf, &file_stat) == 0)
      {
	fd = open(buf, O_WRONLY | O_TRUNC);
	if(fd >= 0)
	  is_open = 1;
      }

      if(!is_open)
	{
	  cout << "Can't open file '" << buf << "' for writing!" << endl;
	  enabled = false;
	}
    }

  // Draw the ball.
  TVector ballPos = ws.getBall().getPosition();
  PutFloat(ballPos.getX()/52.5);
  PutFloat(ballPos.getY()/34.0);
  PutFloatByte(ballPos.getC());


  // Draw the players
  const vector<PlayerState>& tmVec = ws.getTeammates();
  const vector<PlayerState>& oppVec = ws.getOpponents();
  PutByte(1 + tmVec.size() + oppVec.size());

  PutPlayer(ws.getAgent(), 1, 1);
  for(unsigned int i = 0; i < tmVec.size(); i++)
    PutPlayer(tmVec[i], 1, 0);

  for(unsigned int i = 0; i < oppVec.size(); i++)
    PutPlayer(oppVec[i], 2, 0);


  // Draw arrows if any.
  PutByte(arrowStart.size());
  for(unsigned int i = 0; i < arrowStart.size(); i++)
  {
    PutFloat(arrowStart[i].getX()/52.5);
    PutFloat(arrowStart[i].getY()/34.0);
    PutFloat(arrowEnd[i].getX()/52.5);
    PutFloat(arrowEnd[i].getY()/34.0);
    PutByte(arrowDuration[i]);
    PutByte(arrowColor[i]);
  }
  arrowStart.clear();
  arrowEnd.clear();
  arrowDuration.clear();
  arrowColor.clear();


  // Draw circles if any.
  PutByte(circleCenters.size());
  for(unsigned int i = 0; i < circleCenters.size(); i++)
  {
    PutFloat(circleCenters[i].getX()/52.5);
    PutFloat(circleCenters[i].getY()/34.0);
    PutFloat(circleRadii[i]/100.0);
    PutByte(circleDuration[i]);
    PutByte(circleColor[i]);
  }

  circleCenters.clear();
  circleRadii.clear();
  circleDuration.clear();
  circleColor.clear();

  // FIXME: Status not implemented in monitor yet
//   // Print status.
//   file << "(status "
//        << ws.getAgent().getUniformNumber() << " "
//        << (int)(ws.getAgent().getStamina()) << ")";

  // FIXME: Messages not implemented in monitor yet
//   // Print messages.
//   for(unsigned int i = 0; i < messages.size(); i++)
//     {
//       file << "(message \""
// 	   << messages[i] << "\")";
//     }
  messages.clear();
#endif
}

RS_END_NAMESPACE
