/* -*- Mode: C++ -*-
 * $Id: EventView.cc,v 1.3 2001/07/28 07:18:05 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: EventView.cc
 * Unitname: Framework/view
 * $Revision: 1.3 $
 * Created by: Mikael Brnnstrm, Andreas Sigfridsson, Andreas Bckert
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/28 07:18:05 $
 *
 *
 * HISTORY
 *
 * $Log: EventView.cc,v $
 * Revision 1.3  2001/07/28 07:18:05  mikbr212
 * Removed warnings!!
 *
 * Revision 1.2  2001/04/22 14:27:01  andbo928
 * Fixed alot of stuff.
 *
 * Revision 1.1  2001/04/11 18:06:57  andbo928
 * Added eventview.
 *
 * Revision 1.1  2001/04/11 09:10:25  mikbr212
 * Created view
 *
 *
 */

/**
 * @file EventView.cc
 * Implementation file for the class EventView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.3 $
 * $Author: mikbr212 $
 * $Date: 2001/07/28 07:18:05 $
 */

#include "EventView.h"

RS_BEGIN_NAMESPACE

EventView::EventView(const ViewId id)
  : View(id),
    worldStateView(NULL)
{
  getView(worldStateView, WORLD_STATE_VIEW_ID);
}

EventView::~EventView()
{
  viewManager->releaseView(WORLD_STATE_VIEW_ID);
}

void EventView::addEvent(Predicate* _predicate,
			 unsigned int _startCycle,
			 unsigned int _endCycle,
			 void* _appPtr,
			 callbackFn _callback)
{
  predicate.push_back(_predicate);
  startCycle.push_back(_startCycle);
  endCycle.push_back( _endCycle);
  callback.push_back( _callback);
  appPtr.push_back( _appPtr);
}

void EventView::myUpdateAfterInit()
{
}

// NewCycle is called *before* the values in CSensorInterface
// are erased, so this is the last chance to use them...
void EventView::myNewCycle() 
{
  vector<Predicate*>::iterator predIt = predicate.begin();
  vector<unsigned int>::iterator startIt = startCycle.begin();
  vector<unsigned int>::iterator endIt = endCycle.begin();
  vector<callbackFn>::iterator callIt = callback.begin();
  vector<void*>::iterator appIt = appPtr.begin();

  // FIXME!
  unsigned int currentTime = (unsigned int)worldStateView->getWorldState().getGameTime();
  //  int currentTime = 0;
  while(predIt != predicate.end())
    {
      FuzzyBool res;
      if( *endIt < currentTime
		 || (*startIt <= currentTime 
		     && (res = (*predIt)->evaluate()).isTrue()) ) {

	callbackFn cb = *callIt;
	cb(*predIt, res, *appIt);
	
	predIt = predicate.erase(predIt);
	startIt = startCycle.erase(startIt);
	endIt = endCycle.erase(endIt);
	callIt = callback.erase(callIt);
	appIt = appPtr.erase(appIt);
      }  else {
	predIt++;
	startIt++;
	endIt++;
	callIt++;
	appIt++;
      }
    }
}

void EventView::myUpdateAfterSee()
{
}

void EventView::myUpdateAfterHear()
{
}

void EventView::myUpdateAfterCommand()
{
  /* Since we currently send all commands last in the cycle, even
     immediate actions, we can do all the updating at the begining
     of the new cycle. */
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "EventView.icc"
#endif
