/* -*- Mode: C++ -*-
 * $Id: FuzzyPlayerView.h,v 1.1.1.1 2001/04/08 11:33:59 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FuzzyPlayerView.cc
 * Unitname: Framework/view
 * $Revision: 1.1.1.1 $
 * Created by: Mikael Brnnstrm, Andreas Sigfridsson, Andreas Bckert
 * Last modified by $Author: sigge $ $Date: 2001/04/08 11:33:59 $
 *
 *
 * HISTORY
 *
 * $Log: FuzzyPlayerView.h,v $
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.2  2000/11/19 16:46:49  mikbr212
 * now uses absolute vectors
 *
 * Revision 1.1  2000/11/19 11:38:50  mikbr212
 * created
 *
 */

/**
 * @file FuzzyPlayerView.h
 * Implementation file for the class FuzzyPlayerView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:59 $
 */



#ifndef FUZZYPLAYERVIEW_H
#define FUZZYPLAYERVIEW_H

#include "ViewWithHistory.h"
#include "ParameterView.h"
#include "AgentView.h"
#include "GameView.h"
#include "CommandView.h"
#include "FuzzyPlayer.h"
#include "view_constants.h"

RS_BEGIN_NAMESPACE

#define NUMBER_OF_PLAYERS 22

/**
 * @class FuzzyPlayerView FuzzyPlayerView.h
 * This view is used to keep track of the ball.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:59 $
 */
class FuzzyPlayerView : public ViewWithHistory {
public:
  /** Create a ball view with ViewId @a id and the
    history length @a hist_len. */
  FuzzyPlayerView(const ViewId id,
		const UInt hist_len = DEFAULT_HISTORY_LENGTH);

  /** Destroys the ball view. */
  virtual ~FuzzyPlayerView();
  
  /** Callback function. */
  void processPlayer(const ServerPlayerObject& plr);

  /** Get a pointer to the closest player @a hist cycles ago. */
  FuzzyPlayer* getClosestPlayer(const Int hist);

  /** Get a pointer to the closest opponent @a hist cycles ago. */
  FuzzyPlayer* getClosestOpponent(const Int hist);

  /** Get a pointer to the closest teammate @a hist cycles ago. */
  FuzzyPlayer* getClosestTeammate(const Int hist);

  /** Get a pointer to the closest unknown player @a hist cycles ago. */
  FuzzyPlayer* getClosestUnknownPlayer(const Int hist);
  
  // Warning temporary hack!
  /** Get a vector with the opponents (a copy of the actual data). */
  std::vector<FuzzyPlayer> getOpponents(const Int hist) const;

  /** Get a vector with the teammates (a copy of the actual data). */
  std::vector<FuzzyPlayer> getTeammates(const Int hist) const;

  /** Get a vector with the unknown players (a copy of the actual data). */
  std::vector<FuzzyPlayer> getUnknownPlayers(const Int hist) const;

  /** Get a vector with the unknown players (a copy of the actual data). */
  std::vector<FuzzyPlayer> getAllPlayers(const Int hist) const;

protected:
  /** A pointer to the parameter view. */
  ParameterView* parameterView;

  /** A pointer to the game view. */
  //GameView* gameView;

  /** A pointer to the command view. */
  // CommandView* commandView;

  /** A pointer to the agent view. */
  AgentView* agentView;

  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();

  PointUCO clearPointConf(PointUCO &p);
  VectorUCO clearVectorConf(VectorUCO &v);

private:

  std::vector< FuzzyPlayer > playersObs;
  std::vector< std::vector<FuzzyPlayer> > playersHist;

  // Hide copy-constructor and assignment.
  FuzzyPlayerView(const FuzzyPlayerView&);
  FuzzyPlayerView& operator= (const FuzzyPlayerView&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "FuzzyPlayerView.icc"
#endif


#endif // !FUZZYPLAYERVIEW_H

