/* -*- Mode: C++ -*-
 * $Id: Role.h,v 1.6 2001/07/28 16:22:39 mikbr212 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.6 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/28 16:22:39 $
 *
 *
 * HISTORY
 *
 * $Log: Role.h,v $
 * Revision 1.6  2001/07/28 16:22:39  mikbr212
 * Fixed init positions on goal etc
 *
 * Revision 1.5  2001/07/28 13:00:19  mikbr212
 * Changed role
 *
 *
 */

/**
 * @file Role.h
 * Definition file for the class Role.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.6 $
 * $Author: mikbr212 $
 * $Date: 2001/07/28 16:22:39 $
 */

#ifndef __ROLE_H_
#define __ROLE_H_

#include "TVector.h"
#include "WorldStateView.h"
#include "StatisticsView.h"
#include "GameView.h"
#include "ParameterView.h"

RS_BEGIN_NAMESPACE

/** 
 * @class Role Role.h
 * Text...
 *
 * $Revision: 1.6 $
 * $Author: mikbr212 $
 * $Date: 2001/07/28 16:22:39 $
 */
class Role {
  // Nested types
public:
  enum RoleType {
    rtUnknown = 0,
    rtGoalie = 1,
    rtDefensive = 2,
    rtMid = 3,
    rtOffensive = 4
  };

  // Methods / attributes
public:
  
  Role(); 
  ~Role();
  
  static void init();
  static Role getAgentRole();
  static Role getPlayerRole(const PlayerState&);
  
  RoleType getRoleType() const;
  TVector getHomePosition() const;
  float getZoneRadius() const;
  TVector getInitPosition() const;

protected:
  Role(int _uNum);

  static TVector getHomePosition(int uNum);
  static TVector getHomePosition(int uNum, const TVector& ballPos);
  static TVector getInitPosition(int uNum);
  static float getZoneRadius(int uNum);
  static RoleType getRoleType(int uNum);
  
  int uNum;

  static WorldStateView* worldStateView;
  static StatisticsView* statisticsView;
  static GameView* gameView;
  static ParameterView* parameterView;

};

RS_END_NAMESPACE

#endif /* __ROLE_H_ */
