/* -*- Mode: C++ -*-
 * $Id: Stub.cc,v 1.1.1.1 2001/04/08 11:33:59 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Stub.cc
 * Unitname: Library | BasicSystem | Framework | RoboSoc
 * $Revision: 1.1.1.1 $
 * Created by: Fredrik Heintz 2000-08-22
 * Last modified by $Author: sigge $ $Date: 2001/04/08 11:33:59 $
 *
 *
 * HISTORY
 *
 * $Log: Stub.cc,v $
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.2  2000/09/03 11:40:17  frehe
 * Updated the naming of headerfiles and moved the invariant to Stub.icc.
 *
 * Revision 1.1.1.1  2000/08/31 13:26:03  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1  2000/08/22 13:33:55  robocup
 * Made many minor changes. Started to do the complete refactoring and documentation effort. [frehe]
 */

/**
 * @file Stub.cc
 * This is a stub for definition files that all RoboSoc definition files
 * should use. A definition file contains all the non-inline definitions
 * declared by a class or a unit.
 *
 * @par Unit:
 *      Library | BasicSystem | Framework | RoboSoc
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:59 $
 */


#include "Stub.h"
#include "rs_assert.h"


RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

Stub::Stub()
  : attributeName(),
    attributePointer()
{
  RS_ASSERT(invariant());
}

Stub::Stub(const Stub& obj)
  : attributeName(obj.attributeName),
    attributePointer(new TheType(obj.attributePointer))
{
  RS_ASSERT(invariant());
}


// --- DESTRUCTOR ---

Stub::~Stub()
{
  RS_ASSERT(invariant());
  delete attributePointer;
}


// --- OPERATORS ---

Stub& Stub::operator=(const Stub& obj)
{
  if ( &obj != this ) {
    // Clean up the old data.
    delete attributePointer;
    // Copy the new data from obj
    attributeName = obj.attributeName;
    attributePointer = new TheType(obj.attributePointer);
  }
  else {
  }
  
  RS_ASSERT(invariant());
  return *this;
}


// --- OTHER METHODS ---
    
TheType Stub::methodName(const TheType const* arg) const throw(MyException)
{  
  RS_ASSERT(invariant());
  return a_value;
}


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "stub.icc"
#endif
